
"""
    This file contains a set of functional tests designed to check the correct execution of the pipeline and the
    different functions available in Clinica
"""

__author__ = "Mauricio Diaz"
__copyright__ = "Copyright 2016-2019 The Aramis Lab Team"
__credits__ = ["Mauricio Diaz"]
__license__ = "See LICENSE.txt file"
__version__ = "0.2.0"
__maintainer__ = "Mauricio Diaz"
__email__ = "mauricio.diaz@inria.fr"
__status__ = "Development"


import pytest

def pytest_addoption(parser):
    parser.addoption(
            "--working_directory", 
        action="store", 
        help="Working directory for tests"
    )


@pytest.fixture
def cmdopt(request):
    return request.config.getoption("--working_directory")




from os.path import dirname, join, abspath
from setuptools import setup, find_packages
try: # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError: # for pip <= 9.0.3
    from pip.req import parse_requirements

with open(join(dirname(__file__), 'clinica/VERSION'), 'rb') as f:
    version = f.read().decode('ascii').strip()

this_directory = abspath(dirname(__file__))
with open(join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

install_reqs = parse_requirements('requirements.txt', session='hack')
reqs = [str(ir.req) for ir in install_reqs]

setup(
    name='clinica',
    version=version,
    url='http://www.clinica.run',
    description='Software platform for clinical neuroimaging studies',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='ARAMIS Lab',
    maintainer='Clinica developers',
    maintainer_email='clinica-user@inria.fr',
    license='MIT license',
    packages=find_packages(exclude=('tests', 'tests.*')),
    include_package_data=True,
    zip_safe=False,
    entry_points={
        'console_scripts': ['clinica = clinica.cmdline:execute']
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Operating System :: OS Independent',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
    ],
    install_requires=reqs
)



# coding: utf8


def t1_b0_registration(
        participant_id, session_id,
        caps_directory, working_directory=None,
        name="t1_b0_registration"):
    """
    Perform rigid registration of the T1-weighted image onto the B0 image.

    Given a subject, this pipelines performs a registration between its
    T1-weighted image onto its 1mm upsampled B0 image. Once done, the estimated
    transformation matrix is used to align the binary mask of the segmentation
    of the white matter (obtained with FSL fast) from the anatomical space to
    the diffusion space. The same operation is performed on the Desikan and
    Destrieux parcellation (obtained with FreeSurfer recon-all) except that
    these latter are not resliced on the B0 image.

    These steps enable to prepare the data for the DWI processing pipelines.

    Args:
         participant_id (str): Subject (participant) ID in a BIDS format
            ('sub-<participant_label>').
         session_id (str): Session ID in a BIDS format ('ses-<session_label>').
         caps_directory (str): Directory where the results are stored
            in a CAPS hierarchy.
         working_directory (Optional[str]): Directory where the temporary
            results are stored. If not specified, it is automatically generated
            (generally in /tmp/).
         name (Optional[str]): Name of the pipelines.

    Inputnode:
        in_bias_corrected_bet_t1 (str): File containing the bias corrected
            brain extracted T1-weighted image. It corresponds to the
            out_brain_extracted file from FSL-T1 pipelines.
        in_preprocessed_dwi (str): File containing the preprocessed DWI
            dataset. It assumes that the reference b0 is the first volume in
            the dataset (which is the case if you are using Clinica).
        in_b0_mask (str): File containing the mask of the b0 image. It assumes
            that `in_b0_mask` has the same header as `in_preprocessed_dwi`
            (which is the case if you are using Clinica).
        in_white_matter_binary_mask (str): File containing the binary
            segmentation of the white matter (obtained with FSL fast). You can
            use the one generated by FreeSurfer recon-all but you must convert
            your image in FreeSurfer space to Native space first.
        in_desikan_parcellation (str): File containing the Desikan parcellation
            (obtained with FreeSurfer recon-all). The file is usually located
            in ${participant_id}/mri/aparc+aseg.mgz.
        in_destrieux_parcellation (str): File containing the Destrieux
            parcellation (obtained with FreeSurfer recon-all). The file is
            usually located in ${participant_id}/mri/aparc.a2009s+aseg.mgz.

    Outputnode:
        out_registered_t1 (str): File containing the registration of the
            T1-weighted image onto the diffusion space.
        out_flirt_matrix (str): File containing the transformation matrix
            estimated by FSL flirt.
        out_wm_mask_in_diffusion_space (str): File containing the segmentation
            of the white matter in diffusion space.
        out_mrtrix_matrix (str): File containing the transformation matrix in
            MRtrix format (can be used with the mrtransform command).
        out_desikan_in_diffusion_space (str): File containing the Desikan
            parcellation in diffusion space.
        out_destrieux_in_diffusion_space (str): File containing the Desikan
            parcellation in diffusion space.

    """
    from os.path import join
    import nipype.interfaces.fsl as fsl
    import nipype.interfaces.io as nio
    import nipype.interfaces.utility as niu
    import nipype.pipeline.engine as pe
    from nipype.interfaces.freesurfer.preprocess import MRIConvert
    from clinica.utils.freesurfer import freesurfer_volume_to_native_volume
    from clinica.utils.mri_registration import convert_flirt_transformation_to_mrtrix_transformation
    from clinica.utils.mri_registration import apply_mrtrix_transform_without_resampling
    import tempfile
    from clinica.utils.check_dependency import check_freesurfer, check_fsl, check_mrtrix

    if working_directory is None:
        working_directory = tempfile.mkdtemp()

    check_freesurfer()
    check_fsl()
    check_mrtrix()

    caps_identifier = participant_id + '_' + session_id

    inputnode = pe.Node(niu.IdentityInterface(
        fields=['in_bias_corrected_bet_t1', 'in_preprocessed_dwi',
                'in_b0_mask', 'in_white_matter_binary_mask',
                'in_desikan_parcellation', 'in_destrieux_parcellation']),
        name='inputnode')

    get_b0 = pe.Node(fsl.ExtractROI(args='0 1'), name='get_b0')

    upsample_b0 = pe.Node(
        MRIConvert(vox_size=(1, 1, 1), out_type='niigz'),
        name='upsample_b0')

    upsample_b0_mask = pe.Node(
        MRIConvert(vox_size=(1, 1, 1), out_type='niigz'),
        name='upsample_b0_mask')

    registration_t1_to_b0 = pe.Node(fsl.FLIRT(
        dof=6, interp='spline', cost='normmi', cost_func='normmi',
        out_matrix_file=caps_identifier + '_t1-to-b0_withResampling.mat'),
        name='registration_t1_to_b0')

    apply_flirt_registration = pe.Node(
        fsl.ApplyXfm(apply_xfm=True, interp='spline'),
        name='apply_flirt_registration')
    apply_flirt_registration.inputs.out_file = \
        caps_identifier + '_binarymask-whitematter_reslicedOnDiffusionSpace.nii.gz'

    convert_flirt_to_mrtrix = pe.Node(interface=niu.Function(
        input_names=['in_source_image', 'in_reference_image',
                     'in_flirt_matrix', 'name_output_matrix'],
        output_names=['out_mrtrix_matrix'],
        function=convert_flirt_transformation_to_mrtrix_transformation),
        name='convert_flirt_to_mrtrix')
    convert_flirt_to_mrtrix.inputs.name_output_matrix = \
        caps_identifier + '_t1-to-b0_withoutResampling.mat'

    desikan_in_native_space = pe.Node(interface=niu.Function(
        input_names=['freesurfer_volume', 'native_volume', 'name_output_volume'],
        output_names=['out_volume'],
        function=freesurfer_volume_to_native_volume),
        name='desikan_in_native_space')
    destrieux_in_native_space = pe.Node(interface=niu.Function(
        input_names=['freesurfer_volume', 'native_volume', 'name_output_volume'],
        output_names=['out_volume'],
        function=freesurfer_volume_to_native_volume),
        name='destrieux_in_native_space')

    desikan_in_diffusion_space = pe.Node(interface=niu.Function(
        input_names=['in_image', 'in_mrtrix_matrix', 'name_output_image'],
        output_names=['out_deformed_image'],
        function=apply_mrtrix_transform_without_resampling),
        name='desikan_in_diffusion_space')
    desikan_in_diffusion_space.inputs.name_output_image = \
        caps_identifier + '_parcellation-desikan_onDiffusionSpace.nii.gz'
    destrieux_in_diffusion_space = pe.Node(interface=niu.Function(
        input_names=['in_image', 'in_mrtrix_matrix', 'name_output_image'],
        output_names=['out_deformed_image'],
        function=apply_mrtrix_transform_without_resampling),
        name='destrieux_in_diffusion_space')
    destrieux_in_diffusion_space.inputs.name_output_image = \
        caps_identifier + '_parcellation-destrieux_onDiffusionSpace.nii.gz'

    outputnode = pe.Node(niu.IdentityInterface(
        fields=['out_registered_t1', 'out_flirt_matrix',
                'out_wm_mask_in_diffusion_space', 'out_mrtrix_matrix',
                'out_desikan_in_diffusion_space',
                'out_destrieux_in_diffusion_space']),
        name='outputnode')

    datasink = pe.Node(nio.DataSink(), name='datasink')
    datasink.inputs.base_directory = join(caps_directory, 'subjects',
                                          participant_id, session_id)
#    datasink.inputs.substitutions = [('fast_pve_0.nii.gz', caps_identifier + '_binary-csf.nii.gz')]

    wf = pe.Workflow(name=name)
    wf.base_dir = working_directory
    wf.connect([
        # Get b0 from DWI:
        (inputnode, get_b0, [('in_preprocessed_dwi', 'in_file')]),
        # Upsample at 1mm the b0 image:
        (get_b0, upsample_b0, [('roi_file', 'in_file')]),
        # Upsample at 1mm the b0 mask:
        (inputnode, upsample_b0_mask, [('in_b0_mask', 'in_file')]),
        # Register the T1 image onto the b0:
        (inputnode,        registration_t1_to_b0, [('in_bias_corrected_bet_t1', 'in_file')]),
        (upsample_b0,      registration_t1_to_b0, [('out_file', 'reference')]),
        (upsample_b0_mask, registration_t1_to_b0, [('out_file', 'ref_weight')]),
        # Apply flirt registration to WM mask:
        (inputnode,             apply_flirt_registration, [('in_white_matter_binary_mask', 'in_file')]),
        (upsample_b0,           apply_flirt_registration, [('out_file', 'reference')]),
        (registration_t1_to_b0, apply_flirt_registration, [('out_matrix_file', 'in_matrix_file')]),
        # Convert flirt matrix to MRtrix matrix:
        (inputnode,             convert_flirt_to_mrtrix, [('in_bias_corrected_bet_t1', 'in_source_image')]),
        (upsample_b0,           convert_flirt_to_mrtrix, [('out_file', 'in_reference_image')]),
        (registration_t1_to_b0, convert_flirt_to_mrtrix, [('out_matrix_file', 'in_flirt_matrix')]),
        # Convert FreeSurfer parcellations into native space:
        (inputnode, desikan_in_native_space, [('in_desikan_parcellation', 'freesurfer_volume'),
                                              ('in_bias_corrected_bet_t1', 'native_volume')]),
        (inputnode, destrieux_in_native_space, [('in_destrieux_parcellation', 'freesurfer_volume'),
                                                ('in_bias_corrected_bet_t1', 'native_volume')]),
        # Apply registration without resampling on Desikan & Destrieux parcellations:
        (desikan_in_native_space, desikan_in_diffusion_space, [('out_volume', 'in_image')]),  # noqa
        (convert_flirt_to_mrtrix, desikan_in_diffusion_space, [('out_mrtrix_matrix', 'in_mrtrix_matrix')]),  # noqa
        (destrieux_in_native_space, destrieux_in_diffusion_space, [('out_volume', 'in_image')]),  # noqa
        (convert_flirt_to_mrtrix,   destrieux_in_diffusion_space, [('out_mrtrix_matrix', 'in_mrtrix_matrix')]),  # noqa
        # Outputnode:
        (registration_t1_to_b0,        outputnode, [('out_file',                          'out_registered_t1')]),  # noqa
        (registration_t1_to_b0,        outputnode, [('out_matrix_file',                    'out_flirt_matrix')]),  # noqa
        (apply_flirt_registration,     outputnode, [('out_file',             'out_wm_mask_in_diffusion_space')]),  # noqa
        (convert_flirt_to_mrtrix,      outputnode, [('out_mrtrix_matrix',                 'out_mrtrix_matrix')]),  # noqa
        (desikan_in_diffusion_space,   outputnode, [('out_deformed_image',   'out_desikan_in_diffusion_space')]),  # noqa
        (destrieux_in_diffusion_space, outputnode, [('out_deformed_image', 'out_destrieux_in_diffusion_space')]),  # noqa
        # Datasink:
        (registration_t1_to_b0,        datasink, [('out_file',                          'dwi.@out_registered_t1')]),  # noqa
        (registration_t1_to_b0,        datasink, [('out_matrix_file',                    'dwi.@out_flirt_matrix')]),  # noqa
        (apply_flirt_registration,     datasink, [('out_file',              'dwi.@out_wm_mask_in_diffusion_mask')]),  # noqa
        (convert_flirt_to_mrtrix,      datasink, [('out_mrtrix_matrix',                 'dwi.@out_mrtrix_matrix')]),  # noqa
        (desikan_in_diffusion_space,   datasink, [('out_deformed_image',   'dwi.@out_desikan_in_diffusion_space')]),  # noqa
        (destrieux_in_diffusion_space, datasink, [('out_deformed_image', 'dwi.@out_destrieux_in_diffusion_space')])   # noqa
    ])
    return wf



# coding: utf8

import clinica.pipelines.engine as cpe

__author__ = "Jorge Samper-Gonzalez"
__copyright__ = "Copyright 2016-2019 The Aramis Lab Team"
__credits__ = ["Jorge Samper-Gonzalez"]
__license__ = "See LICENSE.txt file"
__version__ = "0.1.0"
__maintainer__ = "Jorge Samper-Gonzalez"
__email__ = "jorge.samper-gonzalez@inria.fr"
__status__ = "Development"


class PETVolume(cpe.Pipeline):
    """PETVolume - Pre-processing of PET images using SPM.

    Args:
        input_dir: A BIDS directory.
        output_dir: An empty output directory where CAPS structured data will be written.
        subjects_sessions_list: The Subjects-Sessions list file (in .tsv format).

    Returns:
        A clinica pipeline object containing the PETVolume pipeline.

    Raises:


    """

    def __init__(self, bids_directory=None, caps_directory=None, tsv_file=None, name=None, group_id='default',
                 fwhm_tsv=None):
        from pandas.io.parsers import read_csv
        import os

        super(PETVolume, self).__init__(bids_directory, caps_directory, tsv_file, name)

        if not group_id.isalnum():
            raise ValueError('Not valid group_id value. It must be composed only by letters and/or numbers')

        # Check that group already exists
        if not os.path.exists(os.path.join(os.path.abspath(caps_directory), 'groups', 'group-' + group_id)):
            error_message = group_id \
                            + ' does not exists, please choose an other one (or maybe you need to run t1-spm-dartel).' \
                            + '\nGroups that already exists in your CAPS directory are : \n'
            list_groups = os.listdir(os.path.join(os.path.abspath(caps_directory), 'groups'))
            is_empty = True
            for e in list_groups:
                if e.startswith('group-'):
                    error_message += e + ' \n'
                    is_empty = False
            if is_empty is True:
                error_message += 'NO GROUP FOUND'
            raise ValueError(error_message)

        self._group_id = group_id
        self._suvr_region = ''
        self._fwhm = None
        self._apply_pvc = False

        if fwhm_tsv is not None:
            if not os.path.isfile(fwhm_tsv):
                raise FileNotFoundError('Could not find the fwhm_tsv file ' + str(fwhm_tsv))
            try:
                fwhm_df = read_csv(fwhm_tsv, sep='\t')
            except (IOError, UnicodeDecodeError):
                raise RuntimeError('An error while reading '
                                   + str(fwhm_tsv) + ' happened')

            if fwhm_df.shape[0] != len(self.subjects):
                raise ValueError('The number of rows in fwhm_tsv file must match the number of subject-session pairs.')

            if any(elem not in ['participant_id', 'session_id', 'fwhm_x', 'fwhm_y', 'fwhm_z'] for elem in list(fwhm_df.columns)):
                raise IOError('The file ' + str(fwhm_tsv)
                              + ' must contains the following columns (separated by tabulation : participant_id, session_id, fwhm_x, fwhm_y, fwhm_z), but we found '
                              + str(list(fwhm_df.columns)) + '. Pay attention to the spaces (there should be none !)')

            subjects_fwhm = list(fwhm_df.participant_id)
            sessions_fwhm = list(fwhm_df.session_id)
            idx_reordered = []
            for i, sub in enumerate(self.subjects):
                current_ses = self.sessions[i]
                idx_sub = [j for j in range(len(subjects_fwhm)) if sub == subjects_fwhm[j] and current_ses == sessions_fwhm[j]]
                if len(idx_sub) == 0:
                    raise RuntimeError('Subject ' + sub + ' with session ' + current_ses + ' that you want to proceed was not found in the PSF specifications ' + str(fwhm_tsv))
                if len(idx_sub) > 1:
                    raise RuntimeError('Subject ' + sub + ' with session ' + current_ses + ' were found multiple times in ' + str(fwhm_tsv))
                idx_reordered.append(idx_sub[0])

            fwhm_x = list(fwhm_df.fwhm_x)
            fwhm_y = list(fwhm_df.fwhm_y)
            fwhm_z = list(fwhm_df.fwhm_z)
            self._fwhm = [[fwhm_x[i], fwhm_y[i], fwhm_z[i]] for i in idx_reordered]
            self._apply_pvc = True

        # Default parameters
        self._parameters = {'pet_type': 'fdg',
                            'mask_tissues': [1, 2, 3],
                            'mask_threshold': 0.3,
                            'pvc_mask_tissues': [1, 2, 3],
                            'smooth': [8],
                            'atlas_list': ['AAL2', 'LPBA40', 'Neuromorphometrics', 'AICHA', 'Hammers']
                            }

    def check_custom_dependencies(self):
        """Check dependencies that can not be listed in the `info.json` file.
        """
        pass

    def get_input_fields(self):
        """Specify the list of possible inputs of this pipelines.

        Returns:
            A list of (string) input fields name.
        """

        return ['pet_image',
                't1_image_native',
                'mask_tissues',
                'pvc_mask_tissues',
                'fwhm',
                'flow_fields',
                'dartel_template',
                'reference_mask']

    def get_output_fields(self):
        """Specify the list of possible outputs of this pipelines.

        Returns:
            A list of (string) output fields name.
        """

        return ['pet_t1_native',
                'pet_mni',
                'pet_suvr',
                'binary_mask',
                'pet_suvr_masked',
                'pet_suvr_masked_smoothed',
                'pet_pvc',
                'pet_pvc_mni',
                'pet_pvc_suvr',
                'pet_pvc_suvr_masked',
                'pet_pvc_suvr_masked_smoothed',
                'atlas_statistics',
                'pvc_atlas_statistics'
                ]

    def build_input_node(self):
        """Build and connect an input node to the pipelines.
        """

        import nipype.interfaces.utility as nutil
        import nipype.pipeline.engine as npe
        import nipype.interfaces.io as nio
        import clinica.pipelines.pet_volume.pet_volume_utils as utils
        from os.path import join, split, realpath

        iterables_fwhm = self._fwhm
        if not self._apply_pvc:
            iterables_fwhm = [[]] * len(self.subjects)

        iterables_node = npe.Node(name="LoadingCLIArguments",
                                  interface=nutil.IdentityInterface(fields=['subject_id', 'session_id', 'fwhm'],
                                                                    mandatory_inputs=True))
        iterables_node.iterables = [('subject_id', self.subjects),
                                    ('session_id', self.sessions),
                                    ('fwhm', iterables_fwhm)]

        iterables_node.synchronize = True

        # PET DataGrabber
        # ===============
        pet_bids_reader = npe.Node(nio.DataGrabber(infields=['subject_id', 'session', 'subject_repeat',
                                                             'session_repeat'],
                                                   outfields=['out_files']), name='pet_bids_reader')
        pet_bids_reader.inputs.base_directory = self.bids_directory
        pet_bids_reader.inputs.template = '%s/%s/pet/%s_%s_task-rest_acq-' + self.parameters['pet_type'] + '_pet.nii*'
        pet_bids_reader.inputs.sort_filelist = False

        # Native T1 DataGrabber
        # ======================
        t1_bids_reader = npe.Node(nio.DataGrabber(infields=['subject_id', 'session', 'subject_repeat',
                                                            'session_repeat'],
                                                  outfields=['out_files']), name='t1_bids_reader')
        t1_bids_reader.inputs.base_directory = self.bids_directory
        t1_bids_reader.inputs.template = '%s/%s/anat/%s_%s_T1w.nii*'
        t1_bids_reader.inputs.sort_filelist = False

        # Flow Fields DataGrabber
        # ========================
        flowfields_caps_reader = npe.Node(nio.DataGrabber(infields=['subject_id', 'session', 'subject_repeat',
                                                                    'session_repeat'],
                                                          outfields=['out_files']), name='flowfields_caps_reader')
        flowfields_caps_reader.inputs.base_directory = join(self.caps_directory, 'subjects')
        flowfields_caps_reader.inputs.template = '%s/%s/t1/spm/dartel/group-' + self._group_id + '/%s_%s_T1w_target-' \
                                                 + self._group_id + '_transformation-forward_deformation.nii*'
        flowfields_caps_reader.inputs.sort_filelist = False

        # Dartel Template DataGrabber
        # ============================
        template_caps_reader = npe.Node(nio.DataGrabber(outfields=['out_files']),
                                        name="template_caps_reader")
        template_caps_reader.inputs.base_directory = self.caps_directory
        template_caps_reader.inputs.template = 'groups/group-' + self._group_id + '/t1/group-' + self._group_id \
                                               + '_template.nii*'
        template_caps_reader.inputs.sort_filelist = False

        # Reference Mask DataGrabber
        # ===========================
        reference_mask = npe.Node(nio.DataGrabber(outfields=['out_files']), name='reference_mask')
        reference_mask.inputs.base_directory = join(split(realpath(__file__))[0], '../../resources/masks')
        reference_mask.inputs.sort_filelist = False
        # TODO ADD DIFFERENT PET TYPES TO PROCESS
        if self.parameters['pet_type'] == 'fdg':
            reference_mask.inputs.template = 'region-pons_eroded-6mm_mask.nii*'
            self._suvr_region = 'pons'
        elif self.parameters['pet_type'] == 'av45':
            reference_mask.inputs.template = 'region-cerebellumPons_eroded-6mm_mask.nii*'
            self._suvr_region = 'cerebellumPons'
        else:
            raise NotImplementedError('Unknown type of PET image. We currently accept as input only "fdg" or "av45"' +
                                      ' as values.')

        # Tissues DataGrabber
        # ====================
        tissue_names = {1: 'graymatter',
                        2: 'whitematter',
                        3: 'csf',
                        4: 'bone',
                        5: 'softtissue',
                        6: 'background'
                        }

        tissues_caps_reader = npe.Node(nio.DataGrabber(infields=['subject_id', 'session', 'subject_repeat',
                                                                 'session_repeat', 'tissues'],
                                                       outfields=['out_files']), name='tissues_caps_reader')
        tissues_caps_reader.inputs.base_directory = join(self.caps_directory, 'subjects')
        tissues_caps_reader.inputs.template = '%s/%s/t1/spm/segmentation/normalized_space/%s_%s_T1w_segm-%s_space-Ixi549Space_modulated-off_probability.nii*'
        tissues_caps_reader.inputs.tissues = [tissue_names[t] for t in self.parameters['mask_tissues']]
        tissues_caps_reader.inputs.sort_filelist = False

        n_tissues = len(self.parameters['mask_tissues'])

        self.connect([(iterables_node, pet_bids_reader, [('subject_id', 'subject_id'),
                                                         ('session_id', 'session'),
                                                         ('subject_id', 'subject_repeat'),
                                                         ('session_id', 'session_repeat')]),
                      (iterables_node, t1_bids_reader, [('subject_id', 'subject_id'),
                                                        ('session_id', 'session'),
                                                        ('subject_id', 'subject_repeat'),
                                                        ('session_id', 'session_repeat')]),
                      (iterables_node, tissues_caps_reader, [(('subject_id', utils.expand_into_list, n_tissues),
                                                              'subject_id'),
                                                             (('session_id', utils.expand_into_list, n_tissues),
                                                              'session'),
                                                             (('subject_id', utils.expand_into_list, n_tissues),
                                                              'subject_repeat'),
                                                             (('session_id', utils.expand_into_list, n_tissues),
                                                              'session_repeat')]),
                      (iterables_node, flowfields_caps_reader, [('subject_id', 'subject_id'),
                                                                ('session_id', 'session'),
                                                                ('subject_id', 'subject_repeat'),
                                                                ('session_id', 'session_repeat')]),
                      (pet_bids_reader, self.input_node, [('out_files', 'pet_image')]),
                      (t1_bids_reader, self.input_node, [('out_files', 't1_image_native')]),
                      (tissues_caps_reader, self.input_node, [('out_files', 'mask_tissues')]),
                      (flowfields_caps_reader, self.input_node, [('out_files', 'flow_fields')]),
                      (template_caps_reader, self.input_node, [('out_files', 'dartel_template')]),
                      (reference_mask, self.input_node, [('out_files', 'reference_mask')]),
                      (iterables_node, self.input_node, [('fwhm', 'fwhm')])
                      ])
        if self._apply_pvc:
            pvc_tissues_caps_reader = npe.Node(
                nio.DataGrabber(infields=['subject_id', 'session', 'subject_repeat', 'session_repeat', 'tissues'],
                                outfields=['out_files']), name='pvc_tissues_caps_reader')
            pvc_tissues_caps_reader.inputs.base_directory = join(self.caps_directory, 'subjects')
            pvc_tissues_caps_reader.inputs.template = '%s/%s/t1/spm/segmentation/native_space/%s_%s_T1w_segm-%s_probability.nii*'
            pvc_tissues_caps_reader.inputs.tissues = [tissue_names[t] for t in self.parameters['pvc_mask_tissues']]
            pvc_tissues_caps_reader.inputs.sort_filelist = False

            n_pvc_tissues = len(self.parameters['pvc_mask_tissues'])

            self.connect([(iterables_node, pvc_tissues_caps_reader, [(('subject_id', utils.expand_into_list,
                                                                       n_pvc_tissues), 'subject_id'),
                                                                     (('session_id', utils.expand_into_list,
                                                                       n_pvc_tissues), 'session'),
                                                                     (('subject_id', utils.expand_into_list,
                                                                       n_pvc_tissues), 'subject_repeat'),
                                                                     (('session_id', utils.expand_into_list,
                                                                       n_pvc_tissues), 'session_repeat')]),
                          (pvc_tissues_caps_reader, self.input_node, [('out_files', 'pvc_mask_tissues')])
                          ])
        else:
            self.input_node.inputs.pvc_mask_tissues = []

    def build_output_node(self):
        """Build and connect an output node to the pipelines.
        """

        import nipype.interfaces.utility as nutil
        import nipype.pipeline.engine as npe
        import nipype.interfaces.io as nio
        from clinica.utils.io import zip_nii
        from clinica.utils.io import fix_join
        import re
        import clinica.pipelines.pet_volume.pet_volume_utils as utils

        # Find container path from pet filename
        # =====================================
        container_path = npe.Node(nutil.Function(input_names=['pet_filename'],
                                                 output_names=['container'],
                                                 function=utils.pet_container_from_filename),
                                  name='container_path')
        container_path.inputs.threshold = self.parameters['mask_threshold']

        # Writing all images into CAPS
        # ============================
        write_images_node = npe.Node(name='write_caps_node',
                                     interface=nio.DataSink())
        write_images_node.inputs.base_directory = self.caps_directory
        write_images_node.inputs.parameterization = False
        write_images_node.inputs.regexp_substitutions = [
            (r'(.*/)pet_t1_native/r(sub-.*)(\.nii(\.gz)?)$', r'\1\2_space-T1w_pet\3'),
            (r'(.*/)pet_pvc/pvc-rbv_r(sub-.*)(\.nii(\.gz)?)$', r'\1\2_space-T1w_pvc-rbv_pet\3'),
            (r'(.*/)pet_mni/wr(sub-.*)(\.nii(\.gz)?)$', r'\1\2_space-Ixi549Space_pet\3'),
            (r'(.*/)pet_pvc_mni/wpvc-rbv_r(sub-.*)(\.nii(\.gz)?)$', r'\1\2_space-Ixi549Space_pvc-rbv_pet\3'),
            (r'(.*/)pet_suvr/suvr_wr(sub-.*)(\.nii(\.gz)?)$',
             r'\1\2_space-Ixi549Space_suvr-' + re.escape(self._suvr_region) + r'_pet\3'),
            (r'(.*/)pet_pvc_suvr/suvr_wpvc-rbv_r(sub-.*)(\.nii(\.gz)?)$',
             r'\1\2_space-Ixi549Space_pvc-rbv_suvr-' + re.escape(self._suvr_region) + r'_pet\3'),
            (r'(.*/)pet_suvr_masked/masked_suvr_wr(sub-.*)(\.nii(\.gz)?)$',
             r'\1\2_space-Ixi549Space_suvr-' + re.escape(self._suvr_region) + r'_mask-brain_pet\3'),
            (r'(.*/)pet_pvc_suvr_masked/masked_suvr_wpvc-rbv_r(sub-.*)(\.nii(\.gz)?)$',
             r'\1\2_space-Ixi549Space_pvc-rbv_suvr-' + re.escape(self._suvr_region) + r'_mask-brain_pet\3'),
            (r'(.*/)pet_suvr_masked_smoothed/(fwhm-[0-9]+mm)_masked_suvr_wr(sub-.*)(\.nii(\.gz)?)$',
             r'\1\3_space-Ixi549Space_suvr-' + re.escape(self._suvr_region) + r'_mask-brain_\2_pet\4'),
            (r'(.*/)pet_pvc_suvr_masked_smoothed/(fwhm-[0-9]+mm)_masked_suvr_wpvc-rbv_r(sub-.*)(\.nii(\.gz)?)$',
             r'\1\3_space-Ixi549Space_pvc-rbv_suvr-' + re.escape(self._suvr_region) + r'_mask-brain_\2_pet\4'),
            (r'(.*/)binary_mask/(sub-.*_T1w_).*(space-[a-zA-Z0-9]+).*(_brainmask\.nii(\.gz)?)$', r'\1\2\3\4')
        ]

        # Writing atlas statistics into CAPS
        # ==================================
        write_atlas_node = npe.Node(name='write_atlas_node',
                                    interface=nio.DataSink())
        write_atlas_node.inputs.base_directory = self.caps_directory
        write_atlas_node.inputs.parameterization = False
        write_atlas_node.inputs.regexp_substitutions = [
            (r'(.*/atlas_statistics/)suvr_wr(sub-.*)(_statistics\.tsv)$', r'\1\2' + r'_suvr-' +
             re.escape(self._suvr_region) + r'\3'),
            (r'(.*/)pvc_(atlas_statistics/)suvr_wpvc-rbv_r(sub-.*)(_statistics\.tsv)$', r'\1\2\3' + r'_pvc-rbv_suvr-' +
             re.escape(self._suvr_region) + r'\4')
        ]

        self.connect([(self.input_node, container_path, [('pet_image', 'pet_filename')]),
                      (container_path, write_images_node, [(('container', fix_join, 'group-' + self._group_id),
                                                            'container')]),
                      (self.output_node, write_images_node, [(('pet_t1_native', zip_nii, True), 'pet_t1_native'),
                                                             (('pet_mni', zip_nii, True), 'pet_mni'),
                                                             (('pet_suvr', zip_nii, True), 'pet_suvr'),
                                                             (('binary_mask', zip_nii, True), 'binary_mask'),
                                                             (('pet_suvr_masked', zip_nii, True), 'pet_suvr_masked'),
                                                             (('pet_suvr_masked_smoothed', zip_nii, True),
                                                              'pet_suvr_masked_smoothed'),
                                                             (('pet_pvc', zip_nii, True), 'pet_pvc'),
                                                             (('pet_pvc_mni', zip_nii, True), 'pet_pvc_mni'),
                                                             (('pet_pvc_suvr', zip_nii, True), 'pet_pvc_suvr'),
                                                             (('pet_pvc_suvr_masked', zip_nii, True),
                                                              'pet_pvc_suvr_masked'),
                                                             (('pet_pvc_suvr_masked_smoothed', zip_nii, True),
                                                              'pet_pvc_suvr_masked_smoothed')]),
                      (container_path, write_atlas_node, [(('container', fix_join, 'group-' + self._group_id),
                                                           'container')]),
                      (self.output_node, write_atlas_node, [('atlas_statistics', 'atlas_statistics'),
                                                            ('pvc_atlas_statistics', 'pvc_atlas_statistics')])
                      ])

    def build_core_nodes(self):
        """Build and connect the core nodes of the pipelines.
        """

        import nipype.interfaces.spm as spm
        import nipype.interfaces.spm.utils as spmutils
        from nipype.interfaces.petpvc import PETPVC
        import nipype.interfaces.utility as nutil
        import nipype.pipeline.engine as npe
        from clinica.utils.io import unzip_nii
        import clinica.pipelines.pet_volume.pet_volume_utils as utils

        import os

        if 'SPMSTANDALONE_HOME' in os.environ:
            if 'MCR_HOME' in os.environ:
                matlab_cmd = (
                        os.path.join(
                            os.environ['SPMSTANDALONE_HOME'], 'run_spm12.sh')
                        + ' ' + os.environ['MCR_HOME']
                        + ' script')
                spm.SPMCommand.set_mlab_paths(matlab_cmd=matlab_cmd, use_mcr=True)

        # Unzipping
        # ==================
        unzip_pet_image = npe.Node(nutil.Function(input_names=['in_file'],
                                                  output_names=['out_file'],
                                                  function=unzip_nii),
                                   name='unzip_pet_image')

        unzip_t1_image_native = npe.Node(nutil.Function(input_names=['in_file'],
                                                        output_names=['out_file'],
                                                        function=unzip_nii),
                                         name='unzip_t1_image_native')

        unzip_flow_fields = npe.Node(nutil.Function(input_names=['in_file'],
                                                    output_names=['out_file'],
                                                    function=unzip_nii),
                                     name='unzip_flow_fields')

        unzip_dartel_template = npe.Node(nutil.Function(input_names=['in_file'],
                                                        output_names=['out_file'],
                                                        function=unzip_nii),
                                         name='unzip_dartel_template')

        unzip_reference_mask = npe.Node(nutil.Function(input_names=['in_file'],
                                                       output_names=['out_file'],
                                                       function=unzip_nii),
                                        name='unzip_reference_mask')

        unzip_mask_tissues = npe.MapNode(nutil.Function(input_names=['in_file'],
                                                        output_names=['out_file'],
                                                        function=unzip_nii),
                                         name='unzip_mask_tissues',
                                         iterfield=['in_file'])

        # Coregister PET into T1 native space
        # ===================================
        coreg_pet_t1 = npe.Node(spm.Coregister(), name='coreg_pet_t1')

        # Spatially normalize PET into MNI
        # ================================
        dartel_mni_reg = npe.Node(spm.DARTELNorm2MNI(), name='dartel_mni_reg')
        dartel_mni_reg.inputs.modulate = False
        dartel_mni_reg.inputs.fwhm = 0

        # Reslice reference region mask into PET
        # ======================================
        reslice = npe.Node(spmutils.Reslice(), name='reslice')

        # Normalize PET values according to reference region
        # ==================================================
        norm_to_ref = npe.Node(nutil.Function(input_names=['pet_image', 'region_mask'],
                                              output_names=['suvr_pet_path'],
                                              function=utils.normalize_to_reference),
                               name='norm_to_ref')

        # Create binary mask from segmented tissues
        # =========================================
        binary_mask = npe.Node(nutil.Function(input_names=['tissues', 'threshold'],
                                              output_names=['out_mask'],
                                              function=utils.create_binary_mask),
                               name='binary_mask')
        binary_mask.inputs.threshold = self.parameters['mask_threshold']

        # Mask PET image
        # ==============
        apply_mask = npe.Node(nutil.Function(input_names=['image', 'binary_mask'],
                                             output_names=['masked_image_path'],
                                             function=utils.apply_binary_mask),
                              name='apply_mask')

        # Smoothing
        # =========
        if self.parameters['smooth'] is not None and len(self.parameters['smooth']) > 0:
            smoothing_node = npe.MapNode(spm.Smooth(),
                                         name='smoothing_node',
                                         iterfield=['fwhm', 'out_prefix'])
            smoothing_node.inputs.fwhm = [[x, x, x] for x in self.parameters['smooth']]
            smoothing_node.inputs.out_prefix = ['fwhm-' + str(x) + 'mm_' for x in self.parameters['smooth']]
            self.connect([
                (apply_mask, smoothing_node, [('masked_image_path', 'in_files')]),
                (smoothing_node, self.output_node, [('smoothed_files', 'pet_suvr_masked_smoothed')])
            ])
        else:
            self.output_node.inputs.pet_suvr_masked_smoothed = [[]]

        # Atlas Statistics
        # ================
        atlas_stats_node = npe.MapNode(nutil.Function(input_names=['in_image',
                                                                   'in_atlas_list'],
                                                      output_names=['atlas_statistics'],
                                                      function=utils.atlas_statistics),
                                       name='atlas_stats_node',
                                       iterfield=['in_image'])
        atlas_stats_node.inputs.in_atlas_list = self.parameters['atlas_list']

        # Connection
        # ==========
        self.connect([(self.input_node, unzip_pet_image, [('pet_image', 'in_file')]),
                      (self.input_node, unzip_t1_image_native, [('t1_image_native', 'in_file')]),
                      (self.input_node, unzip_flow_fields, [('flow_fields', 'in_file')]),
                      (self.input_node, unzip_dartel_template, [('dartel_template', 'in_file')]),
                      (self.input_node, unzip_reference_mask, [('reference_mask', 'in_file')]),
                      (self.input_node, unzip_mask_tissues, [('mask_tissues', 'in_file')]),

                      (unzip_pet_image, coreg_pet_t1, [('out_file', 'source')]),
                      (unzip_t1_image_native, coreg_pet_t1, [('out_file', 'target')]),
                      (unzip_flow_fields, dartel_mni_reg, [('out_file', 'flowfield_files')]),
                      (unzip_dartel_template, dartel_mni_reg, [('out_file', 'template_file')]),
                      (unzip_reference_mask, reslice, [('out_file', 'in_file')]),
                      (unzip_mask_tissues, binary_mask, [('out_file', 'tissues')]),

                      (coreg_pet_t1, dartel_mni_reg, [('coregistered_source', 'apply_to_files')]),
                      (dartel_mni_reg, reslice, [('normalized_files', 'space_defining')]),
                      (dartel_mni_reg, norm_to_ref, [('normalized_files', 'pet_image')]),
                      (reslice, norm_to_ref, [('out_file', 'region_mask')]),
                      (norm_to_ref, apply_mask, [('suvr_pet_path', 'image')]),
                      (binary_mask, apply_mask, [('out_mask', 'binary_mask')]),
                      (norm_to_ref, atlas_stats_node, [('suvr_pet_path', 'in_image')]),

                      (coreg_pet_t1, self.output_node, [('coregistered_source', 'pet_t1_native')]),
                      (dartel_mni_reg, self.output_node, [('normalized_files', 'pet_mni')]),
                      (norm_to_ref, self.output_node, [('suvr_pet_path', 'pet_suvr')]),
                      (binary_mask, self.output_node, [('out_mask', 'binary_mask')]),
                      (apply_mask, self.output_node, [('masked_image_path', 'pet_suvr_masked')]),
                      (atlas_stats_node, self.output_node, [('atlas_statistics', 'atlas_statistics')])
                      ])

        # PVC
        # ==========
        if self._apply_pvc:
            # Unzipping
            # =========
            unzip_pvc_mask_tissues = npe.MapNode(nutil.Function(input_names=['in_file'],
                                                                output_names=['out_file'],
                                                                function=unzip_nii),
                                                 name='unzip_pvc_mask_tissues',
                                                 iterfield=['in_file'])

            # Creating Mask to use in PVC
            # ===========================
            pvc_mask = npe.Node(nutil.Function(input_names=['tissues'],
                                               output_names=['out_mask'],
                                               function=utils.create_pvc_mask),
                                name='pvc_mask')
            # PET PVC
            # =======
            petpvc = npe.Node(PETPVC(), name='pvc')
            petpvc.inputs.pvc = 'RBV'
            petpvc.inputs.out_file = 'pvc.nii'

            # Spatially normalize PET into MNI
            # ================================
            dartel_mni_reg_pvc = npe.Node(spm.DARTELNorm2MNI(), name='dartel_mni_reg_pvc')
            dartel_mni_reg_pvc.inputs.modulate = False
            dartel_mni_reg_pvc.inputs.fwhm = 0

            # Reslice reference region mask into PET
            # ======================================
            reslice_pvc = npe.Node(spmutils.Reslice(), name='reslice_pvc')

            # Normalize PET values according to reference region
            # ==================================================
            norm_to_ref_pvc = npe.Node(nutil.Function(input_names=['pet_image', 'region_mask'],
                                                      output_names=['suvr_pet_path'],
                                                      function=utils.normalize_to_reference),
                                       name='norm_to_ref_pvc')

            # Mask PET image
            # ==============
            apply_mask_pvc = npe.Node(nutil.Function(input_names=['image', 'binary_mask'],
                                                     output_names=['masked_image_path'],
                                                     function=utils.apply_binary_mask),
                                      name='apply_mask_pvc')
            # Smoothing
            # =========
            if self.parameters['smooth'] is not None and len(self.parameters['smooth']) > 0:
                smoothing_pvc = npe.MapNode(spm.Smooth(),
                                            name='smoothing_pvc',
                                            iterfield=['fwhm', 'out_prefix'])
                smoothing_pvc.inputs.fwhm = [[x, x, x] for x in self.parameters['smooth']]
                smoothing_pvc.inputs.out_prefix = ['fwhm-' + str(x) + 'mm_' for x in self.parameters['smooth']]
                self.connect([
                    (apply_mask_pvc, smoothing_pvc, [('masked_image_path', 'in_files')]),
                    (smoothing_pvc, self.output_node, [('smoothed_files', 'pet_pvc_suvr_masked_smoothed')])
                ])
            else:
                self.output_node.inputs.pet_pvc_suvr_masked_smoothed = [[]]
            # Atlas Statistics
            # ================
            atlas_stats_pvc = npe.MapNode(nutil.Function(input_names=['in_image',
                                                                      'in_atlas_list'],
                                                         output_names=['atlas_statistics'],
                                                         function=utils.atlas_statistics),
                                          name='atlas_stats_pvc',
                                          iterfield=['in_image'])
            atlas_stats_pvc.inputs.in_atlas_list = self.parameters['atlas_list']

            # Connection
            # ==========
            self.connect([(self.input_node, unzip_pvc_mask_tissues, [('pvc_mask_tissues', 'in_file')]),
                          (unzip_pvc_mask_tissues, pvc_mask, [('out_file', 'tissues')]),
                          (unzip_flow_fields, dartel_mni_reg_pvc, [('out_file', 'flowfield_files')]),
                          (unzip_dartel_template, dartel_mni_reg_pvc, [('out_file', 'template_file')]),
                          (unzip_reference_mask, reslice_pvc, [('out_file', 'in_file')]),
                          (coreg_pet_t1, petpvc, [('coregistered_source', 'in_file'),
                                                  (('coregistered_source', utils.pet_pvc_name, 'RBV'), 'out_file')]),
                          (pvc_mask, petpvc, [('out_mask', 'mask_file')]),
                          (self.input_node, petpvc, [(('fwhm', utils.get_from_list, 0), 'fwhm_x'),
                                                     (('fwhm', utils.get_from_list, 1), 'fwhm_y'),
                                                     (('fwhm', utils.get_from_list, 2), 'fwhm_z')]),
                          (petpvc, dartel_mni_reg_pvc, [('out_file', 'apply_to_files')]),
                          (dartel_mni_reg_pvc, reslice_pvc, [('normalized_files', 'space_defining')]),
                          (dartel_mni_reg_pvc, norm_to_ref_pvc, [('normalized_files', 'pet_image')]),
                          (reslice_pvc, norm_to_ref_pvc, [('out_file', 'region_mask')]),

                          (norm_to_ref_pvc, apply_mask_pvc, [('suvr_pet_path', 'image')]),
                          (binary_mask, apply_mask_pvc, [('out_mask', 'binary_mask')]),
                          (norm_to_ref_pvc, atlas_stats_pvc, [('suvr_pet_path', 'in_image')]),

                          (petpvc, self.output_node, [('out_file', 'pet_pvc')]),
                          (dartel_mni_reg_pvc, self.output_node, [('normalized_files', 'pet_pvc_mni')]),
                          (norm_to_ref_pvc, self.output_node, [('suvr_pet_path', 'pet_pvc_suvr')]),
                          (apply_mask_pvc, self.output_node, [('masked_image_path', 'pet_pvc_suvr_masked')]),
                          (atlas_stats_pvc, self.output_node, [('atlas_statistics', 'pvc_atlas_statistics')])
                          ])
        else:
            self.output_node.inputs.pet_pvc = [[]]
            self.output_node.inputs.pet_pvc_mni = [[]]
            self.output_node.inputs.pet_pvc_suvr = [[]]
            self.output_node.inputs.pet_pvc_suvr_masked = [[]]
            self.output_node.inputs.pvc_atlas_statistics = [[]]
            self.output_node.inputs.pet_pvc_suvr_masked_smoothed = [[]]



# coding: utf8


"""This module contains utilities to check dependencies of the different
neuroimaging tools."""


def is_binary_present(binary):
    """
    Check if a binary is present.

    This function checks if the program is present. Do not use this function
    with a binary GUI, it will open the GUI.

    Taken from:
    https://stackoverflow.com/questions/11210104/check-if-a-program-exists-from-a-python-script

    Args:
        binary (str): Name of the program.

    Returns:
        True if the binary is present, False otherwise.
    """
    import subprocess
    import os

    try:
        devnull = open(os.devnull)
        subprocess.Popen([binary], stdout=devnull, stderr=devnull).communicate()
    except OSError as e:
        if e.errno == os.errno.ENOENT:
            return False
#    cprint("Binary '%s' has been detected." % binary)
    return True


def check_ants():
    """
    Check ANTs software.

    This function checks if ANTs is present (ANTSPATH, binaries, & scripts).
    """
    import os
    from clinica.utils.stream import cprint

    try:
        antspath = os.environ.get('ANTSPATH', '')
        if not antspath:
            raise RuntimeError('ANTSPATH variable is not set')
    except Exception as e:
        cprint(str(e))

    list_binaries = ['N4BiasFieldCorrection', 'antsRegistrationSyNQuick.sh']
    for binary in list_binaries:
        if not is_binary_present(binary):
            raise RuntimeError(
                '%s from ANTs Software is not present '
                'in your PATH environment.' % binary)

    cprint('ANTs has been detected')


def check_freesurfer():
    """
    Check FreeSurfer software.

    This function checks if FreeSurfer is present (FREESURFER_HOME & binaries).
    """
    import os
    from clinica.utils.stream import cprint

    try:
        freesurfer_home = os.environ.get('FREESURFER_HOME', '')
        if not freesurfer_home:
            raise RuntimeError('FREESURFER_HOME variable is not set')
    except Exception as e:
        cprint(str(e))

    list_binaries = ['mri_convert', 'recon-all']
    for binary in list_binaries:
        if not is_binary_present(binary):
            raise RuntimeError(
                '%s from FreeSurfer Software is not present in your PATH '
                'environment: did you source ${FREESURFER_HOME}/'
                'SetUpFreeSurfer.sh ?' % binary)

    cprint('FreeSurfer has been detected')


def check_fsl():
    """
    Check FSL software.

    This function checks if FSL is present (FSLDIR & binaries) and if the
    version of FSL is recent.
    """
    import os
    import nipype.interfaces.fsl as fsl

    from clinica.utils.stream import cprint

    try:
        fsl_dir = os.environ.get('FSLDIR', '')
        if not fsl_dir:
            raise RuntimeError('FSLDIR variable is not set')
    except Exception as e:
        cprint(str(e))

    try:
        if fsl.Info.version().split(".") < ['5', '0', '5']:
            raise RuntimeError('FSL version must be greater than 5.0.5')
    except Exception as e:
        cprint(str(e))

    list_binaries = ['bet', 'flirt', 'fast', 'first']
    for binary in list_binaries:
        if not is_binary_present(binary):
            raise RuntimeError(
                '%s from FSL Software is not present in your '
                'PATH environment.' % binary)

    cprint('FSL has been detected')


def check_mrtrix():
    """
    Check MRtrix software.

    This function checks if MRtrix is present (MRTRIX_HOME & binaries).
    """
    import os
    from clinica.utils.stream import cprint

    try:
        mrtrix_home = os.environ.get('MRTRIX_HOME', '')
        if not mrtrix_home:
            raise RuntimeError('MRTRIX_HOME variable is not set')
    except Exception as e:
        print(str(e))

    list_binaries = ['transformconvert', 'mrtransform', 'dwi2response', 'tckgen']
    for binary in list_binaries:
        if not is_binary_present(binary):
            raise RuntimeError(
                '%s from MRtrix Software is not present in your '
                'PATH environment.' % binary)

    cprint('MRtrix has been detected')


def check_spm():
    """
    Check SPM software.

    This function checks if SPM is present (SPM_HOME).
    """
    import os
    from clinica.utils.stream import cprint

    try:
        spm_home = os.environ.get('SPM_HOME', '')
        if not spm_home:
            raise RuntimeError('SPM_HOME variable is not set')
    except Exception as e:
        print(str(e))

    # list_binaries = ['matlab']
    # for binary in list_binaries:
    #     if not is_binary_present(binary):
    #         raise RuntimeError(
    #             '%s from SPM Software is not present in your '
    #             'PATH environment.' % binary)

    cprint('SPM has been detected')


def check_matlab():
    """
    Check matlab toolbox.

    This function checks if matlab is present (matlab for linux and MATLABCMD for mac).
    """
    import os
    import sys
    from clinica.utils.stream import cprint

    if not is_binary_present("matlab"):
        raise RuntimeError('Matlab was not found in PATH environment. Did you add it?')



# coding: utf-8

"""
 Module for converting FLAIR of ADNI
"""

__author__ = "Jorge Samper-Gonzalez and Sabrina Fontanella"
__copyright__ = "Copyright 2016-2019 The Aramis Lab Team"
__license__ = "See LICENSE.txt file"
__version__ = "0.1.0"
__maintainer__ = "Jorge Samper Gonzalez"
__email__ = "jorge.samper-gonzalez@inria.fr"
__status__ = "Development"


def convert_adni_flair(source_dir, csv_dir, dest_dir, subjs_list=None):
    """

    Args:
        source_dir: path to the ADNI directory
        csv_dir: path to the clinical data directory
        dest_dir: path to the destination directory
        subjs_list: subjects list

    Returns:

    """
    import pandas as pd
    from os import path
    from clinica.utils.stream import cprint
    from colorama import Fore

    if subjs_list is None:
        adni_merge_path = path.join(csv_dir, 'ADNIMERGE.csv')
        adni_merge = pd.io.parsers.read_csv(adni_merge_path, sep=',')
        subjs_list = list(adni_merge.PTID.unique())

    cprint('Calculating paths of FLAIR images. Output will be stored in '
           + path.join(dest_dir, 'conversion_info') + '.')
    images = compute_flair_paths(source_dir, csv_dir, dest_dir, subjs_list)
    cprint('Paths of FLAIR images found. Exporting images into BIDS ...')
    flair_paths_to_bids(images, dest_dir)
    cprint(Fore.GREEN + 'FLAIR conversion done.' + Fore.RESET)
    pass


def compute_flair_paths(source_dir, csv_dir, dest_dir, subjs_list):
    """
    Compute paths to FLAIR images to convert to BIDS

    Args:
        source_dir: path to the ADNI directory
        csv_dir: path to the clinical data directory
        dest_dir: path to the destination directory
        subjs_list: subjects list

    Returns:
        images: pandas dataframe that contains the path to all the FLAIR images to convert

    """
    import pandas as pd
    from os import path, walk, mkdir
    from clinica.utils.stream import cprint

    flair_col_df = ['Subject_ID', 'VISCODE', 'Visit', 'Sequence', 'Scan_Date',
                    'Study_ID', 'Series_ID', 'Image_ID', 'Field_Strength', 'Scanner', 'Enhanced']

    flair_df = pd.DataFrame(columns=flair_col_df)
    adni_merge_path = path.join(csv_dir, 'ADNIMERGE.csv')

    ida_meta_path = path.join(csv_dir, 'MRILIST.csv')
    mri_qc_path = path.join(csv_dir, 'MAYOADIRL_MRI_IMAGEQC_12_08_15.csv')

    adni_merge = pd.io.parsers.read_csv(adni_merge_path, sep=',')
    ida_meta = pd.io.parsers.read_csv(ida_meta_path, sep=',')

    ida_meta = ida_meta[ida_meta.SEQUENCE.map(lambda x: x.lower().find('flair') > -1)]
    mri_qc = pd.io.parsers.read_csv(mri_qc_path, sep=',')
    mri_qc = mri_qc[mri_qc.series_type == 'AFL']

    for subj in subjs_list:
        # print 'Computing path for subj', subj

        adnimerge_subj = adni_merge[adni_merge.PTID == subj]

        # Sort the values by examination date
        adnimerge_subj = adnimerge_subj.sort_values('EXAMDATE')
        ida_meta_subj = ida_meta[ida_meta.SUBJECT == subj]
        ida_meta_subj = ida_meta_subj.sort_values('SCANDATE')

        mri_qc_subj = mri_qc[mri_qc.RID == int(subj[-4:])]
        visits = visits_to_timepoints_flair(subj, ida_meta_subj, adnimerge_subj)
        keys = visits.keys()
        # What is supposed to do the line below ????
        # keys.sort()

        for visit_info in visits.keys():
            visit_str = visits[visit_info]
            visit_ida_meta = ida_meta_subj[ida_meta_subj.VISIT == visit_str]
            axial_ida_meta = visit_ida_meta[visit_ida_meta.SEQUENCE.map(lambda x: x.lower().find('enhanced') < 0)]
            axial = flair_image(subj, visit_info[0], visits[visit_info], axial_ida_meta, mri_qc_subj, False)
            row_to_append = pd.DataFrame(axial, index=['i', ])
            flair_df = flair_df.append(row_to_append, ignore_index=True)

    images = flair_df
    is_dicom = []
    nifti_paths = []
    count = 0

    for row in images.iterrows():
        image = row[1]
        seq_path = path.join(source_dir, str(image.Subject_ID), str(image.Sequence))
        count += 1
        series_path = ''
        s = 'S' + str(image.Series_ID)
        for (dirpath, dirnames, filenames) in walk(seq_path):
            found = False
            for d in dirnames:
                if d == s:
                    series_path = path.join(dirpath, d)
                    found = True
                    break
            if found:
                break

        nifti_path = series_path
        dicom = True
        is_dicom.append(dicom)
        nifti_paths.append(nifti_path)

    images.loc[:, 'Is_Dicom'] = pd.Series(is_dicom, index=images.index)
    images.loc[:, 'Path'] = pd.Series(nifti_paths, index=images.index)

    # Drop all the lines that have the Path section empty
    # images = images.drop(images[images.Path == ''].index)
    # Store the paths inside a file called t1_paths inside the input directory

    flair_tsv_path = path.join(dest_dir, 'conversion_info')
    if not path.exists(flair_tsv_path):
        mkdir(flair_tsv_path)

    cprint('\tDone! Saving the results into' + path.join(flair_tsv_path, 'flair_paths.tsv'))
    images.to_csv(path.join(flair_tsv_path, 'flair_paths.tsv'),
                  sep='\t',
                  index=False)
    return images


def flair_paths_to_bids(images, dest_dir, mod_to_update=False):
    """
    Convert FLAIR images

    Args:
        images: dataframe returned by the method compute_flair_paths
        dest_dir: path to the destination directory
        mod_to_update: if is true and an image is already existing it will overwrite the old version

    """
    from multiprocessing import cpu_count, Pool, Value
    from functools import partial

    counter = None

    def init(args):
        ''' store the counter for later use '''
        global counter
        counter = args

    subjs_list = [sub for sub in images['Subject_ID'].unique()
                  if sub == sub and 'S' in sub]

    counter = Value('i', 0)
    partial_generate_subject_files = partial(generate_subject_files,
                                             images=images,
                                             dest_dir=dest_dir,
                                             mod_to_update=mod_to_update)
    poolrunner = Pool(cpu_count(), initializer=init, initargs=(counter,))
    poolrunner.map(partial_generate_subject_files, subjs_list)
    del counter


def generate_subject_files(subj, images, dest_dir, mod_to_update):
    import clinica.iotools.bids_utils as bids
    import clinica.iotools.converters.adni_to_bids.adni_utils as adni_utils
    from clinica.utils.stream import cprint
    import subprocess
    import os
    import shutil
    from os import path
    from glob import glob

    alpha_id = bids.remove_space_and_symbols(subj)
    bids_id = 'sub-ADNI' + alpha_id
    # Extract the list of sessions available from the flair paths files, removing the duplicates
    sess_list = images[(images['Subject_ID'] == subj)]['VISCODE'].unique()

    if not os.path.exists(path.join(dest_dir, bids_id)):
        os.mkdir(path.join(dest_dir, bids_id))

    # For each session available, create the folder if doesn't exist and convert the files
    for ses in sess_list:
        with counter.get_lock():
            counter.value += 1
        cprint('[FLAIR] Processing subject ' + str(subj)
               + ' - session ' + ses + ', ' + str(counter.value)
               + ' / ' + str(len(images)))
        ses_bids = adni_utils.viscode_to_session(ses)
        bids_ses_id = 'ses-' + ses_bids
        bids_file_name = bids_id + '_ses-' + ses_bids
        ses_path = path.join(dest_dir, bids_id, bids_ses_id)
        if mod_to_update:
            if os.path.exists(path.join(ses_path, 'FLAIR')):
                shutil.rmtree(path.join(ses_path, 'FLAIR'))
        if not os.path.exists(ses_path):
            os.mkdir(ses_path)
        flair_info = images[
            (images['Subject_ID'] == subj) & (images['VISCODE'] == ses)]
        # For the same subject, same session there could be multiple flar with different acq label
        for j in range(len(flair_info)):
            flair_subj = flair_info.iloc[j]
            # TODO For now in CLINICA we ignore Enhanced FLAIR.
            if flair_subj['Enhanced']:
                continue
            if type(flair_subj['Path']) != float and flair_subj['Path'] != '':
                if not os.path.exists(path.join(ses_path, 'FLAIR')):
                    os.mkdir(path.join(ses_path, 'FLAIR'))
                flair_path = flair_subj['Path']

                bids_name = bids_file_name + '_FLAIR'
                # bids.dcm_to_nii(dwi_path, path.join(ses_path, 'dwi'), bids_name)

                bids_dest_dir = path.join(ses_path, 'FLAIR')

                if not os.path.exists(bids_dest_dir):
                    os.mkdir(dest_dir)
                command = 'dcm2niix -b y -z y -o ' + bids_dest_dir + ' -f ' + bids_name + ' ' + flair_path
                subprocess.run(command,
                               shell=True,
                               stderr=subprocess.DEVNULL,
                               stdout=subprocess.DEVNULL)

                # If dcm2niix didn't work use dcm2nii
                # print path.join(dest_dir, bids_name + '.nii.gz')
                if not os.path.exists(
                        path.join(bids_dest_dir, bids_name + '.nii.gz')):
                    cprint('\tConversion with dcm2niix failed, trying with dcm2nii')

                    # Find all the files eventually created by dcm2niix and remove them
                    flair_dcm2niix = glob(
                        path.join(bids_dest_dir, bids_name + '*'))
                    for d in flair_dcm2niix:
                        os.remove(d)

                    command = 'dcm2nii -a n -d n -e n -i y -g y -p n -m n -r n -x n -o ' + bids_dest_dir + ' ' + flair_path
                    subprocess.run(command,
                                   shell=True,
                                   stdout=subprocess.DEVNULL,
                                   stderr=subprocess.DEVNULL)
                    nii_file = path.join(bids_dest_dir,
                                         subj.replace('_', '') + '.nii.gz')
                    if os.path.exists(nii_file):
                        os.rename(nii_file, path.join(bids_dest_dir,
                                                      bids_name + '.nii.gz'))
                    else:
                        cprint('WARNING: CONVERSION FAILED...')


def select_image_qc(id_list, mri_qc_subj):
    """

    Args:
        id_list:
        mri_qc_subj:

    Returns:

    """
    import numpy as np

    if len(id_list) == 0:
        return None

    selected_image = None
    image_ids = ['I' + str(imageuid) for imageuid in id_list]
    int_ids = [int(imageuid) for imageuid in id_list]
    images_qc = mri_qc_subj[mri_qc_subj.loni_image.isin(image_ids)]

    if images_qc.shape[0] < 1:
        return min(int_ids)

    if np.sum(images_qc.series_selected) == 1:
        selected_image = images_qc[images_qc.series_selected == 1].iloc[0].loni_image[1:]
    else:
        images_not_rejected = images_qc[images_qc.series_quality < 4]

        if images_not_rejected.shape[0] < 1:

            # There are no images that passed the qc
            # so we'll try to see if there are other images without qc,
            # otherwise return None
            qc_ids = set([int(qc_id[1:]) for qc_id in images_qc.loni_image.unique()])
            no_qc_ids = list(set(int_ids) - qc_ids)

            if len(no_qc_ids) == 0:
                return None
            else:
                return min(no_qc_ids)

        series_quality = [q if q > 0 else 4 for q in list(images_not_rejected.series_quality)]
        best_q = np.amin(series_quality)
        if best_q == 4:
            best_q = -1
        images_best_qc = images_not_rejected[images_not_rejected.series_quality == best_q]
        if images_best_qc.shape[0] == 1:
            selected_image = images_best_qc.iloc[0].loni_image[1:]
        else:
            best_ids = [int(x[1:]) for x in images_best_qc.loni_image.unique()]
            selected_image = min(best_ids)

    return int(selected_image)


def visits_to_timepoints_flair(subject, ida_meta_subj, adnimerge_subj):
    """

    Args:
        subject:
        ida_meta_subj:
        adnimerge_subj:

    Returns:

    """
    from datetime import datetime
    from clinica.iotools.converters.adni_to_bids.adni_utils import days_between
    from clinica.utils.stream import cprint

    visits = dict()
    unique_visits = list(ida_meta_subj.VISIT.unique())
    pending_timepoints = []

    # We try to obtain the corresponding image Visit for a given VISCODE
    for adni_row in adnimerge_subj.iterrows():
        visit = adni_row[1]
        if visit.ORIGPROT == 'ADNI2':
            if visit.VISCODE == 'bl':
                preferred_visit_name = 'ADNI2 Screening MRI-New Pt'
            elif visit.VISCODE == 'm03':
                preferred_visit_name = 'ADNI2 Month 3 MRI-New Pt'
            elif visit.VISCODE == 'm06':
                preferred_visit_name = 'ADNI2 Month 6-New Pt'
            else:
                year = str(int(visit.VISCODE[1:]) / 12)
                preferred_visit_name = 'ADNI2 Year ' + year + ' Visit'
        else:
            if visit.VISCODE == 'bl':
                if visit.ORIGPROT == 'ADNI1':
                    preferred_visit_name = 'ADNI Screening'
                else:  # ADNIGO
                    preferred_visit_name = 'ADNIGO Screening MRI'
            elif visit.VISCODE == 'm03':  # Only for ADNIGO Month 3
                preferred_visit_name = 'ADNIGO Month 3 MRI'
            else:
                month = int(visit.VISCODE[1:])
                if month < 54:
                    preferred_visit_name = 'ADNI1/GO Month ' + str(month)
                else:
                    preferred_visit_name = 'ADNIGO Month ' + str(month)

        if preferred_visit_name in unique_visits:
            key_preferred_visit = (visit.VISCODE, visit.COLPROT, visit.ORIGPROT)
            if key_preferred_visit not in visits.keys():
                visits[key_preferred_visit] = preferred_visit_name
            elif visits[key_preferred_visit] != preferred_visit_name:
                cprint('Multiple visits for one timepoint!')
                cprint(subject)
                cprint(key_preferred_visit)
                cprint(visits[key_preferred_visit])
                cprint(visit)
            unique_visits.remove(preferred_visit_name)
            continue

        pending_timepoints.append(visit)

    # Then for images.Visit non matching the expected labels we find the closest date in visits list
    for visit in unique_visits:
        image = (ida_meta_subj[ida_meta_subj.VISIT == visit]).iloc[0]
        min_db = 100000
        min_db2 = 0
        min_visit = None
        min_visit2 = None

        for timepoint in pending_timepoints:
            db = days_between(image['SCANDATE'], timepoint.EXAMDATE)
            if db < min_db:
                min_db2 = min_db
                min_visit2 = min_visit

                min_db = db
                min_visit = timepoint

        if min_visit is None:
            cprint('No corresponding timepoint in ADNIMERGE for subject ' + subject + ' in visit ' + image.VISIT)
            cprint(image)
            continue

        if min_visit2 is not None and min_db > 90:
            cprint('More than 60 days for corresponding timepoint in ADNIMERGE for subject ' + subject + ' in visit ' + image.VISIT + ' on ' + image.SCANDATE)
            cprint('Timepoint 1: ' + min_visit.VISCODE + ' - ' + min_visit.ORIGPROT + ' on ' + min_visit.EXAMDATE + ' (Distance: ' + str(
                min_db) + ' days)')
            cprint('Timepoint 2: ' + min_visit2.VISCODE + ' - ' + min_visit2.ORIGPROT + ' on ' + min_visit2.EXAMDATE + ' (Distance: ' + str(
                min_db2) + ' days)')

            # If image is too close to the date between two visits we prefer the earlier visit
            if (datetime.strptime(min_visit.EXAMDATE, "%Y-%m-%d")
                    > datetime.strptime(image.SCANDATE, "%Y-%m-%d")
                    > datetime.strptime(min_visit2.EXAMDATE, "%Y-%m-%d")):
                dif = days_between(min_visit.EXAMDATE, min_visit2.EXAMDATE)
                if abs((dif / 2.0) - min_db) < 30:
                    min_visit = min_visit2
            cprint('We prefer ' + min_visit.VISCODE)

        key_min_visit = (min_visit.VISCODE, min_visit.COLPROT, min_visit.ORIGPROT)
        if key_min_visit not in visits.keys():
            visits[key_min_visit] = image.VISIT
        elif visits[key_min_visit] != image.VISIT:
            cprint('Multiple visits for one timepoint!')

    return visits


def flair_image(subject_id, timepoint, visit_str, ida_meta_scans, mri_qc_subj, enhanced):
    """

    Args:
        subject_id:
        timepoint:
        visit_str:
        ida_meta_scans:
        mri_qc_subj:
        enhanced:

    Returns:

    """
    from clinica.iotools.converters.adni_to_bids.adni_utils import replace_sequence_chars

    sel_image = select_image_qc(list(ida_meta_scans.IMAGEUID), mri_qc_subj)
    if sel_image is None:
        return None

    sel_scan = ida_meta_scans[ida_meta_scans.IMAGEUID == sel_image].iloc[0]
    sequence = sel_scan.SEQUENCE
    sequence = replace_sequence_chars(sequence)
    image_dict = {'Subject_ID': subject_id,
                  'VISCODE': timepoint,
                  'Visit': visit_str,
                  'Sequence': sequence,
                  'Scan_Date': sel_scan['SCANDATE'],
                  'Study_ID': str(int(sel_scan.STUDYID)),
                  'Series_ID': str(int(sel_scan.SERIESID)),
                  'Image_ID': str(int(sel_scan.IMAGEUID)),
                  'Field_Strength': sel_scan.MAGSTRENGTH,
                  'Enhanced': enhanced}

    return image_dict



# coding: utf8

from os.path import dirname
from os.path import abspath
from os.path import join as pathjoin

from grabbit import Layout

__all__ = ['CAPSLayout']


class CAPSLayout(Layout):

    def __init__(self, path, config=None, **kwargs):
        if config is None:
            root = dirname(abspath(__file__))
            config = pathjoin(root, 'config', 'caps.json')

        super(CAPSLayout, self).__init__(path, config,
                                         dynamic_getters=True, **kwargs)



# coding: utf8


def permutation_test(vector_1, vector_2, number_of_permutation, tails=2):
    """
    This function take two vectors as entry and return the p_value
    of a permutation test.
    If permutation not done, p_value is set to 1.
    This function performs a two tails permutation on *non-nul values*
    If tails is set to 1 the contrast is supposed to be vector_1 > vector_2

    WARNING: here 0 are not removed from the dataset
    If tails is set to 1 the contrast is supposed to be vector_1 > vector_2
    """
    import numpy as np

    def exact_mc_perm_test(xs, ys, nmc, tails=2):
        n, k = len(xs), 0.
        if tails == 2:
            diff = np.abs(np.mean(xs) - np.mean(ys))
        if tails == 1:
            diff = np.mean(xs) - np.mean(ys)
        zs = np.concatenate([xs, ys])
        for j in range(nmc):
            np.random.shuffle(zs)
            if tails == 2:
                diff_random = np.abs(np.mean(zs[:n]) - np.mean(zs[n:]))
            if tails == 1:
                diff_random = np.mean(zs[:n]) - np.mean(zs[n:])
            k += diff < diff_random
        return k/float(nmc)

    p_value = 1.

    x = vector_1
    y = vector_2

    if x != np.array([]) and y != np.array([]):
        if x.sum() != 0 or y.sum() != 0:
            p_value = exact_mc_perm_test(x, y, number_of_permutation, tails)

    return p_value


def t_test(vector_1, vector_2, tails=2):
    """
    This function takes two vectors as entry and return the p-value
    of a t_test.
    If the t_test not done (only zeros in both vectors), p_value is set to 1.
    This function performs a t_test on *non-nul values*.
    If tails is set to 1 the contrast is supposed to be vector_1 > vector_2.

    WARNING: here 0 are not removed from the dataset
    If tails is set to 1 the contrast is supposed to be vector_1 > vector_2
    """

    import numpy as np
    from scipy.stats import ttest_ind

    p_value = 1.

    X = vector_1
    Y = vector_2

    if X != np.array([]) and Y != np.array([]):
        p_value = ttest_ind(X, Y, equal_var=False).pvalue
        if tails == 1:
            x = np.array(X)
            array_y = np.array(Y)
            if x.mean() > array_y.mean():
                p_value = float(p_value)/2.0
            else:
                p_value = 1 - float(p_value)/2.0

    return p_value


def mann_whitney(vector_1, vector_2, tails=2):
    """
    This function take two vectors as entry and return
    the p_value of a  Mann_Whitney U test.
    If the U test not done (only 0 in both vectors), p_value is set to 1.
    This function performs a U test on *non-nul values*

    WARNING: here 0 are not removed from the dataset
    If tails is set to 1 the contrast is supposed to be vector_1 > vector_2
    """

    import numpy as np
    from scipy.stats import mannwhitneyu

    p_value = 1.

    x = vector_1
    non_zero_x = [x[k] for k in np.nonzero(x)[0].tolist()]

    y = vector_2
    non_zero_y = [y[k] for k in np.nonzero(y)[0].tolist()]

    if x != np.array([]) and y != np.array([]):
        if non_zero_x != np.array([]) or non_zero_y != np.array([]):
            p_value = mannwhitneyu(x, y).pvalue
            if tails == 2:
                p_value = 2*p_value

    return p_value


def fdr_correction_matrix(p_value_matrix, template=None):
    """
    This function take a p value matrix as entry and return the corrected
    p_value for False Rate Discovery.
    If not all statistical tests have been performed (typically in DTI at
    a absent connection) a template matrix (which is a binary metrix with 1
    if the test is performed and 0 else) with the same shape as p_value_matrix
    input of the actually performed test can be provide at input.
    """

    import numpy as np
    from mne.stats import fdr_correction

    if type(template) == type(p_value_matrix):
        if p_value_matrix.shape != template.shape:
            raise IOError(
                'p_value_matrix and template should have the same shape.')

    if type(template) == type(p_value_matrix):
        p_value_corrected = np.ones(p_value_matrix.shape)
        reject_test = np.zeros(p_value_matrix.shape, dtype=bool)
        eff_p_value = []
        index_of_eff_p_value = []
        for i in np.arange(0, p_value_matrix.shape[0]):
            for j in np.arange(0, i):
                if template[j, i] == 1:
                    eff_p_value += [p_value_matrix[j, i]]
                    index_of_eff_p_value += [(j, i)]
        reject, p_corrected = fdr_correction(eff_p_value)
        for i, corrected in enumerate(p_corrected):
            p_value_corrected[
                index_of_eff_p_value[i][0], index_of_eff_p_value[i][1]
            ] = corrected
            reject_test[
                index_of_eff_p_value[i][0], index_of_eff_p_value[i][1]
            ] = reject[i]
    elif not template:
        reject_test, p_value_corrected = fdr_correction(p_value_matrix)
    else:
        raise IOError('template input should be an numpy array or None.')
    return reject_test, p_value_corrected


def create_new_feature_tsv(subjects_visits_tsv, bids_dir, dest_tsv, added_features):
    """
        This func is to add new features(columns) from the subjects_visits_list
        TSV file, and use the generated file in the statistical analysis
        added_features : list of str, the added features that you want to write
        into the tsv file, e.g. ['age_bl', 'sex']

    Args:
        subjects_visits_tsv: tsv files containing just participant_id and session_id columns
        bids_dir: BIDS directory
        dest_tsv: the destination tsv file
        added_features: a list containing all the new columns from the participant.tsv in BIDS directory.

    Returns:

    """
    import pandas as pd
    from os.path import join, isfile
    import logging
    from pandas import concat
    logging.basicConfig(level=logging.DEBUG)

    if not isfile(join(bids_dir, 'participants.tsv')):
        raise Exception('participants.tsv not found')
    sub_set = pd.io.parsers.read_csv(subjects_visits_tsv, sep='\t')
    all_set = pd.io.parsers.read_csv(join(bids_dir, 'participants.tsv'), sep='\t')
    all_set.set_index("participant_id", inplace=True)
    selected_subj = all_set.loc[list(sub_set.participant_id)]
    if sub_set.shape[0] != selected_subj.shape[0]:
        missing_subj = set(list(sub_set.participant_id)) - set(list(selected_subj.participant_id))
        msg = "The missing subjects are %s" % list(missing_subj)
        logging.info(msg)
        raise Exception('There are subjects which are not included in full dataset, please check it out')

    new_features = selected_subj[added_features]
    new_features.reset_index(inplace=True, drop=True)
    all_features = concat([sub_set, new_features], axis=1)
    all_features.to_csv(dest_tsv, sep='\t', index=False, encoding='utf-8')


def statistics_on_atlas(in_normalized_map, in_atlas, out_file=None):
    """
    Compute statistics of a map on an atlas.

    Given an atlas image with a set of ROIs, this function computes the mean of
    a normalized map (e.g. GM segmentation, FA map from DTI, etc.) on each ROI.

    Args:
        in_normalized_map (str): File containing a scalar image registered
            on the atlas.
        in_atlas (:obj: AbstractClass): An atlas with a set of ROI. These ROI
            are used to compute statistics.
        out_file (Optional[str]): Name of the output file.

    Returns:
        out_file (str): TSV file containing the statistics (content of the
            columns: label, mean scalar, std of the scalar', number of voxels).
    """
    from clinica.utils.atlas import AtlasAbstract
    import nibabel as nib
    import numpy as np
    import pandas
    import os.path as op
    from clinica.utils.stream import cprint

    if not isinstance(in_atlas, AtlasAbstract):
        raise Exception("Atlas element must be an AtlasAbstract type")

    if out_file is None:
        fname, ext = op.splitext(op.basename(in_normalized_map))
        if ext == ".gz":
            fname, ext2 = op.splitext(fname)
            ext = ext2 + ext
        out_file = op.abspath("%s_statistics_%s.tsv"
                              % (fname, in_atlas.get_name_atlas()))

    atlas_labels = nib.load(in_atlas.get_atlas_labels())
    atlas_labels_data = atlas_labels.get_data()

    img = nib.load(in_normalized_map)
    img_data = img.get_data()

    atlas_correspondance = pandas.io.parsers.read_csv(in_atlas.get_tsv_roi(), sep='\t')
    label_name = list(atlas_correspondance.roi_name)
    label_value = list(atlas_correspondance.roi_value)  # TODO create roi_value column in lut_*.txt and remove irrelevant RGB information

    mean_signal_value = []
    for label in label_value:
        current_mask_label = atlas_labels_data == label
        masked_data = np.array(img_data, copy=True)
        masked_data[np.invert(current_mask_label)] = 0
        mean_signal_value.append(np.sum(masked_data) / np.sum(current_mask_label))

    try:
        data = pandas.DataFrame({'label_name': label_name,
                                 'mean_scalar': mean_signal_value
                                 })
        data.to_csv(out_file, sep='\t', index=True, encoding='utf-8')
    except Exception as e:
        cprint("Impossible to save %s with pandas" % out_file)
        raise e

    return out_file



# coding: utf8

# WARNING: Don't put any import statement here except if it's absolutely
# necessary. Put it *inside* the different methods.
# Otherwise it will slow down the dynamic loading of the pipelines list by the
# command line tool.
import clinica.pipelines.engine as cpe

__author__ = "Simona Bottani"
__copyright__ = "Copyright 2016-2019 The Aramis Lab Team"
__license__ = "See LICENSE.txt file"
__version__ = "0.1.0"
__maintainer__ = "Simona Bottani"
__email__ = "simona.bottani@icm-institute.org"
__status__ = "Development"


class SpatialSVM(cpe.Pipeline):
    """SpatialSVM - Prepare input data for SVM with spatial and anatomical regularization.

    Todos:
        - [ ] Final version of CAPS.
        - [ ] Remove --voxel_size flag and detect automatically this parameter.

    Args:
        input_dir: A BIDS directory.
        output_dir: An empty output directory where CAPS structured data will be written.
        subjects_sessions_list: The Subjects-Sessions list file (in .tsv format).

    Returns:
        A clinica pipeline object containing the SpatialSVM pipeline.

    Raises:
    """

    def check_custom_dependencies(self):
        """Check dependencies that can not be listed in the `info.json` file.
        """
        pass

    def get_input_fields(self):
        """Specify the list of possible inputs of this pipeline.

        Returns:
            A list of (string) input fields name.
        """

        return ['dartel_input', 'input_image']

    def get_output_fields(self):
        """Specify the list of possible outputs of this pipeline.

        Returns:
            A list of (string) output fields name.
        """

        return ['regularized_image']

    def build_input_node(self):
        """Build and connect an input node to the pipeline.
        """

        import nipype.interfaces.utility as nutil
        import nipype.pipeline.engine as npe
        from clinica.utils.stream import cprint
        from os.path import exists, join, abspath
        from os import listdir

        # Check that group-id already exists
        if not exists(join(abspath(self.caps_directory), 'groups', 'group-' + self.parameters['group_id'])):
            error_message = 'group_id : ' + self.parameters['group_id'] + ' does not exists, ' \
                            + 'please choose an other one. Groups that exist' \
                            + 's in your CAPS directory are : \n'
            list_groups = listdir(join(abspath(self.caps_directory), 'groups'))
            has_one_group = False
            for e in list_groups:
                if e.startswith('group-'):
                    error_message += e + ' \n'
                    has_one_group = True
            if not has_one_group:
                error_message = error_message + 'No group found ! ' \
                                + 'Use t1-volume pipeline if you do not ' \
                                + 'have a template yet ! '
            raise ValueError(error_message)

        read_parameters_node = npe.Node(name="LoadingCLIArguments",
                                        interface=nutil.IdentityInterface(fields=self.get_input_fields(),
                                                                          mandatory_inputs=True))
        image_type = self.parameters['image_type']
        pet_type = self.parameters['pet_type']
        no_pvc = self.parameters['no_pvc']

        participant_id_pycaps = [sub[4:] for sub in self.subjects]
        session_id_pycaps = [ses[4:] for ses in self.sessions]

        input_image = []
        subjects_not_found = []
        if image_type == 't1':
            for i, sub in enumerate(self.subjects):
                input_image_single_subject = join(self.caps_directory,
                                                  'subjects', sub, self.sessions[i], 't1/spm/dartel/group-'
                                                  + self.parameters['group_id'], sub + '_' + self.sessions[i]
                                                  + '_T1w_segm-graymatter_space-Ixi549Space_modulated-on_probability.nii.gz')
                if not exists(input_image_single_subject):
                    subjects_not_found.append(input_image_single_subject)
                else:
                    input_image.append(input_image_single_subject)

        elif image_type is 'pet':
            if no_pvc.lower() == 'true':
                for i, sub in enumerate(self.subjects):

                    input_image_single_subject = join(self.caps_directory,
                                                      'subjects', sub, self.sessions[i], 'pet/preprocessing/group-'
                                                      + self.parameters['group_id'], sub + '_' + self.sessions[i]
                                                      + '_task-rest_acq-' + pet_type + '_pet_space-Ixi549Space_suvr-pons_pet.nii.gz')
                    if not exists(input_image_single_subject):
                        subjects_not_found.append(input_image_single_subject)
                    else:
                        input_image.append(input_image_single_subject)

            elif no_pvc.lower() == 'false':
                subjects_not_found = []
                for i, sub in enumerate(self.subjects):

                    input_image_single_subject = join(self.caps_directory,
                                                      'subjects', sub, self.sessions[i], 'pet/preprocessing/group-'
                                                      + self.parameters['group_id'], sub + '_' + self.sessions[i]
                                                      + '_task-rest_acq-' + pet_type + '_pet_space-Ixi549Space_pvc-rbv_suvr-pons_pet.nii.gz')
                    if not exists(input_image_single_subject):
                        subjects_not_found.append(input_image_single_subject)
                    else:
                        input_image.append(input_image_single_subject)
            else:
                raise ValueError(no_pvc + ' is not a valid keyword for -no_pvc'
                                 + ':only True or False are accepted (string)')
        else:
            raise ValueError('Image type ' + image_type + ' unknown')

        if len(subjects_not_found) > 0:
            error_string = ''
            for file in subjects_not_found:
                error_string = error_string + file + '\n'
            raise IOError('Following files were not found :\n' + error_string)

        if len(input_image) != len(self.subjects):
            raise IOError(str(len(input_image)) + ' file(s) found. ' + str(len(self.subjects)) + ' are expected')

        dartel_input = [join(self.caps_directory,
                             'groups',
                             'group-' + self.parameters['group_id'],
                             't1',
                             'group-' + self.parameters['group_id'] + '_template.nii.gz')]
        if not exists(dartel_input[0]):
            raise IOError('Dartel Input ' + dartel_input[0] + ' does not seem to exist')

        read_parameters_node.inputs.dartel_input = dartel_input
        read_parameters_node.inputs.input_image = input_image

        self.connect([
            (read_parameters_node,      self.input_node,    [('dartel_input',    'dartel_input')]),
            (read_parameters_node,      self.input_node,    [('input_image',    'input_image')])

        ])

    def build_output_node(self):
        """Build and connect an output node to the pipeline.
        """

        # In the same idea as the input node, this output node is supposedly
        # used to write the output fields in a CAPS. It should be executed only
        # if this pipeline output is not already connected to a next Clinica
        # pipeline.

        pass

    def build_core_nodes(self):
        """Build and connect the core nodes of the pipeline.
        """

        import clinica.pipelines.machine_learning_spatial_svm.spatial_svm_utils as utils
        import nipype.interfaces.utility as nutil
        import nipype.pipeline.engine as npe
        import nipype.interfaces.io as nio

        fisher_tensor_generation = npe.Node(name="obtain_g_fisher_tensor",
                                            interface=nutil.Function(input_names=['dartel_input', 'FWHM'],
                                                                     output_names=['fisher_tensor', 'fisher_tensor_path'],
                                                                     function=utils.obtain_g_fisher_tensor))
        fisher_tensor_generation.inputs.FWHM = self.parameters['fwhm']

        time_step_generation = npe.Node(name='estimation_time_step',
                                        interface=nutil.Function(input_names=['dartel_input', 'FWHM', 'g'],
                                                                 output_names=['t_step', 'json_file'],
                                                                 function=utils.obtain_time_step_estimation))
        time_step_generation.inputs.FWHM = self.parameters['fwhm']

        heat_solver_equation = npe.MapNode(name='heat_solver_equation',
                                           interface=nutil.Function(input_names=['input_image', 'g',
                                                                                 'FWHM', 't_step', 'dartel_input'],
                                                                    output_names=['regularized_image'],
                                                                    function=utils.heat_solver_equation),
                                           iterfield=['input_image'])
        heat_solver_equation.inputs.FWHM = self.parameters['fwhm']

        datasink = npe.Node(nio.DataSink(),
                            name='sinker')
        datasink.inputs.base_directory = self.caps_directory
        datasink.inputs.parameterization = True
        if self.parameters['image_type'] == 't1':
            datasink.inputs.regexp_substitutions = [
                (r'(.*)/regularized_image/.*/(.*(sub-(.*)_ses-(.*))_T1w(.*)_probability(.*))$',
                 r'\1/subjects/sub-\4/ses-\5/machine_learning/input_spatial_svm/group-' + self.parameters[
                     'group_id'] + r'/\3_T1w\6_spatialregularization\7'),

                (r'(.*)json_file/(output_data.json)$',
                 r'\1/groups/group-' + self.parameters['group_id'] + r'/machine_learning/input_spatial_svm/group-' + self.parameters[
                     'group_id'] + r'_space-Ixi549Space_parameters.json'),

                (r'(.*)fisher_tensor_path/(output_fisher_tensor.npy)$',
                 r'\1/groups/group-' + self.parameters['group_id'] + r'/machine_learning/input_spatial_svm/group-' + self.parameters[
                     'group_id'] + r'_space-Ixi549Space_gram.npy')
            ]

        elif self.parameters['image_type'] == 'pet':
            datasink.inputs.regexp_substitutions = [
                (r'(.*)/regularized_image/.*/(.*(sub-(.*)_ses-(.*))_(task.*)_pet(.*))$',
                 r'\1/subjects/sub-\4/ses-\5/machine_learning/input_spatial_svm/group-' + self.parameters[
                     'group_id'] + r'/\3_\6_spatialregularization\7'),
                (r'(.*)json_file/(output_data.json)$',
                 r'\1/groups/group-' + self.parameters['group_id'] + r'/machine_learning/input_spatial_svm/group-' +
                 self.parameters['group_id'] + r'_space-Ixi549Space_parameters.json'),
                (r'(.*)fisher_tensor_path/(output_fisher_tensor.npy)$',
                 r'\1/groups/group-' + self.parameters['group_id'] + r'/machine_learning/input_spatial_svm/group-' +
                 self.parameters[
                     'group_id'] + r'_space-Ixi549Space_gram.npy')
            ]
        # Connection
        # ==========
        self.connect([
            (self.input_node,      fisher_tensor_generation,    [('dartel_input',    'dartel_input')]),
            (fisher_tensor_generation,      time_step_generation,    [('fisher_tensor',    'g')]),

            (self.input_node, time_step_generation, [('dartel_input', 'dartel_input')]),
            (self.input_node, heat_solver_equation, [('input_image', 'input_image')]),
            (fisher_tensor_generation, heat_solver_equation, [('fisher_tensor', 'g')]),
            (time_step_generation, heat_solver_equation, [('t_step', 't_step')]),
            (self.input_node, heat_solver_equation, [('dartel_input', 'dartel_input')]),

            (fisher_tensor_generation, datasink, [('fisher_tensor_path', 'fisher_tensor_path')]),
            (time_step_generation, datasink, [('json_file', 'json_file')]),
            (heat_solver_equation, datasink, [('regularized_image', 'regularized_image')])
        ])



# coding: utf8

"""Define the command line parser for each utility (pipeline, iotools, etc.)
"""

import abc
from argparse import ArgumentParser
from os.path import join
from os import getcwd
from os.path import expanduser

from colorama import Fore

PIPELINE_CATEGORIES = {
    'CLINICA_COMPULSORY': (Fore.BLUE
                           + 'Clinica mandatory arguments'
                           + Fore.RESET),
    'OPTIONAL': (Fore.BLUE
                 + 'Pipeline options'
                 + Fore.RESET),
    'CLINICA_OPTIONAL': (Fore.BLUE
                         + 'Clinica standard options'
                         + Fore.RESET),
    'ADVANCED': (Fore.BLUE
                 + 'Pipelines advanced options'
                 + Fore.RESET),
    'IOTOOLS_OPTIONS': (Fore.BLUE
                        + 'Optional arguments'
                        + Fore.RESET)
    }


class CmdParser:
    """Abstract class to extend in order to create your command line parser

    For pipelines, please use the 'clinica generate template' command.
    For converters, see clinica/iotools/converters/*/*_cli.py
    For iotools, see clinica/iotools/utils/data_handling_cli.py
    """
    __metaclass__ = abc.ABCMeta

    def __init__(self):
        self.reset()
        self.build()

    def build(self):
        self.define_name()
        self.define_description()
        self.set_content()
        self.define_options()

    def reset(self):
        self._args = ArgumentParser()
        self._name = None

    def set_content(self):
        from colorama import Fore
        self._args._positionals.title = (
                Fore.BLUE
                + 'Mandatory arguments'
                + Fore.RESET
                )
        self._args._optionals.title = (
                Fore.BLUE
                + 'Optional arguments'
                + Fore.RESET
                )
        if self._description is None:
            self._description = self._name
            self._args.description = (
                    Fore.GREEN
                    + ('If you are not familiar with Clinica, see:'
                       'http://clinica.run/doc/InteractingWithClinica/')
                    + Fore.RESET
                    )
        else:
            self._args.description = (
                    (
                        Fore.GREEN
                        + ('%s\n\nIf you are not familiar with Clinica, see:'
                           'http://clinica.run/doc/InteractingWithClinica')
                        + Fore.RESET
                    ) % (self._description)
                )

    @property
    def options(self): return self._args

    @options.setter
    def options(self, x): self._args = x

    @property
    def name(self): return self._name

    @name.setter
    def name(self, x): self._name = x

    @property
    def description(self): return self._description

    @description.setter
    def description(self, x): self._description = x

    @abc.abstractmethod
    def define_name(self): pass

    def define_description(self):
        self._description = None

    @abc.abstractmethod
    def define_options(self): pass

    @abc.abstractmethod
    def run_command(self, args): pass

    def absolute_path(self, arg):
        if arg is None:
            return None
        elif arg[:1] == '~':
            return expanduser(arg)
        elif arg[:2] == './':
            return join(getcwd(), arg[2:])
        else:
            return join(getcwd(), arg)


def init_cmdparser_objects(root_parser, parser, objects):
    """
    Init all derived CmdParser instances with specific data.

    Args:
        root_parser: The root parser
        parser: The ArgParser node (e.g. 'run' or 'convert')
        objects: All CmdParser instances of this file
    """

    def silent_help():
        pass

    def error_message(p):
        def error(x):
            p.print_help()
            root_parser.print_help = silent_help
            exit(-1)
        return error

    def init(x):
        import argparse
        x.options = parser.add_parser(x.name,
                                      add_help=False,
                                      help=x.description,
                                      formatter_class=argparse.RawDescriptionHelpFormatter)
        x.options.error = error_message(x.options)
        x.options.set_defaults(func=x.run_command)
        x.build()

    for x in objects:
        try:
            init(x)
        except BaseException:
            pass


def get_cmdparser_names(objects=None):
    """
    Return the names of all pipelines

    Args:
        objects: All CmdParser instances of this file

    Returns:
        The names of all pipelines
    """
    if objects is None:
        objects = get_cmdparser_objects()
    for x in objects:
        yield x.name


# class CmdParserInsightToBids(CmdParser):
#
#     def define_name(self):
#         self._name = 'insight-to-bids'
#
#     def define_options(self):
#         self._args.add_argument("dataset_directory",
#                                help='Path of the unorganized INSIGHT directory.')
#         self._args.add_argument("bids_directory",
#                                 help='Path to the BIDS directory.')
#         self._args.add_argument("-co", type=bool, default=False,
#                                 help='(Optional) Given an already existing BIDS output folder, convert only the clinical data.')
#
#     def run_command(self, args):
#         from clinica.bids import insight_to_bids
#         insight_to_bids.convert(args.dataset_directory, args.bids_directory)


# class CmdParserPrevDemAlsToBids(CmdParser):
#
#     def define_name(self):
#         self._name = 'prevdemals-to-bids'
#
#     def define_options(self):
#         self._args.add_argument("dataset_directory",
#                                help='Path of the unorganized INSIGHT directory.')
#         self._args.add_argument("bids_directory",
#                                 help='Path to the BIDS directory.')
#         self._args.add_argument("-co", type=bool, default=False,
#                                 help='(Optional) Given an already existing BIDS output folder, convert only the clinical data.
#     def run_command(self, args):
#         from clinica.bids import prevdemals_to_bids
#         prevdemals_to_bids.convert(args.dataset_directory, args.bids_directory)


# class CmdParserHmtcToBids(CmdParser):
#
#     def define_name(self):
#         self._name = 'hmtc-to-bids'
#
#     def define_options(self):
#         self._args.add_argument("dataset_directory",
#                                 help='Path of the unorganized HMTC directory.')
#         self._args.add_argument("bids_directory",
#                                 help='Path to the BIDS directory.')
#
#     def run_command(self, args):
#         from clinica.iotools import hmtc_to_bids
#         hmtc_to_bids.convert(args.dataset_directory, args.bids_directory)



# coding: utf8

import clinica.engine as ce


class T1FreeSurferCrossSectionalCLI(ce.CmdParser):

    def define_name(self):
        """Define the sub-command name to run this pipelines.
        """
        self._name = 't1-freesurfer'

    def define_description(self):
        """Define a description of this pipeline.
        """
        self._description = 'Cross-sectional pre-processing of T1w images with FreeSurfer:\nhttp://clinica.run/doc/Pipelines/T1_FreeSurfer/'

    def define_options(self):
        """Define the sub-command arguments
        """
        from clinica.engine.cmdparser import PIPELINE_CATEGORIES
        # Clinica compulsory arguments (e.g. BIDS, CAPS, group_id)
        clinica_comp = self._args.add_argument_group(PIPELINE_CATEGORIES['CLINICA_COMPULSORY'])
        clinica_comp.add_argument("bids_directory",
                                  help='Path to the BIDS directory.')
        clinica_comp.add_argument("caps_directory",
                                  help='Path to the CAPS directory.')
        # Optional arguments (e.g. FWHM)
        optional = self._args.add_argument_group(PIPELINE_CATEGORIES['OPTIONAL'])
        optional.add_argument("-raa", "--recon_all_args",
                              help='Additional flags for recon-all command line (default: --recon_all_args "-qcache")')
        # Clinica standard arguments (e.g. --n_procs)
        clinica_opt = self._args.add_argument_group(PIPELINE_CATEGORIES['CLINICA_OPTIONAL'])
        clinica_opt.add_argument("-tsv", "--subjects_sessions_tsv",
                                 help='TSV file containing a list of subjects with their sessions.')
        clinica_opt.add_argument("-wd", "--working_directory",
                                 help='Temporary directory to store pipelines intermediate results')
        clinica_opt.add_argument("-np", "--n_procs",
                                 metavar=('N'), type=int,
                                 help='Number of cores used to run in parallel')

    def run_command(self, args):
        """
        Run the pipelines with defined args
        """
        import os
        import datetime
        from colorama import Fore
        from clinica.utils.stream import cprint
        from .t1_freesurfer_cross_sectional_pipeline import T1FreeSurferCrossSectional
        from tempfile import mkdtemp

        pipeline = T1FreeSurferCrossSectional(
            bids_directory=self.absolute_path(args.bids_directory),
            caps_directory=self.absolute_path(args.caps_directory),
            tsv_file=self.absolute_path(args.subjects_sessions_tsv)
        )

        pipeline.parameters = {
            'recon_all_args': args.recon_all_args or '-qcache'
        }

        if args.working_directory is None:
            args.working_directory = mkdtemp()
        pipeline.base_dir = self.absolute_path(args.working_directory)

        if args.n_procs:
            pipeline.run(plugin='MultiProc',
                         plugin_args={'n_procs': args.n_procs})
        else:
            pipeline.run()

        now = datetime.datetime.now().strftime('%H:%M:%S')
        cprint('%s[%s]%s The %s pipeline has completed. You can now delete the working directory (%s).' %
               (Fore.GREEN, now, Fore.RESET, self._name, os.path.join(os.path.abspath(args.working_directory), pipeline.__class__.__name__)))



# coding: utf8

__author__ = "Junhao Wen"
__copyright__ = "Copyright 2016-2019 The Aramis Lab Team"
__credits__ = ["Junhao Wen"]
__license__ = "See LICENSE.txt file"
__version__ = "0.1.0"
__maintainer__ = "Junhao Wen"
__email__ = "Junhao.Wen@inria.fr"
__status__ = "Development"


def bids_datagrabber(input_dir, subject_list, session_list):
    """
    Fetch t1 images from a BIDS directory based on subject_list and a
    session_list

    Args:
        input_dir: BIDS directory
        subject_list: a list containing all the participant_id
        session_list: a list containing all the session_id

    Returns: a list containing all the t1 images
    """
    from bids.grabbids.bids_layout import BIDSLayout
    from clinica.utils.stream import cprint

    bidslayout = BIDSLayout(input_dir)
    anat_t1 = []
    missing_subject_session = []
    if not bidslayout.get(target='run', return_type='id', type='T1w'):
        for i in range(len(subject_list)):
            t1 = bidslayout.get(return_type='file',
                                type='T1w',
                                extensions=['nii|nii.gz'],
                                session=session_list[i].replace('ses-', ''),
                                subject=subject_list[i].replace('sub-', ''))
            if len(t1) == 0:
                missing_subject_session.append([subject_list[i], session_list[i]])
            else:
                anat_t1.append(t1)
    else:
        cprint("There are more than one run for T1w image for this analysis")
        for i in range(len(subject_list)):
            t1 = bidslayout.get(return_type='file',
                                type='T1w',
                                extensions=['nii|nii.gz'],
                                session=session_list[i].replace('ses-', ''),
                                subject=subject_list[i].replace('sub-', ''),
                                run='1')
            if len(t1) == 0:
                missing_subject_session.append([subject_list[i], session_list[i]])
            else:
                anat_t1.append(t1)

    # Check if pybids works well to find all the T1 images
    if len(missing_subject_session) > 0:
        error_string = 'Please verify there is no error in your TSV file. Clinica could not find T1w for those ' + str(len(missing_subject_session)) + ' subjects - session :'
        for e in missing_subject_session:
            error_string += '\n' + e[0] + ' with session ' + e[1]
        raise IOError(error_string)
    if len(anat_t1) != len(subject_list) or len(anat_t1) != len(session_list):
        raise ValueError('Found ' + str(len(anat_t1)) + '  T1w but there are ' + str(len(subject_list)) + ' subjects.')

    return anat_t1


def get_dirs_check_reconalled(output_dir, subject_list, session_list):
    """
    Get the info from subjects_visits_tsv, like subject_dir, subject_id,
    subject_list, session_list. Also, this function checks out the rerun of
    the dataset, if the subject result folder has been created, you should
    check out the result and decide if you are going to rerun it or just
    ignore it.

    Args:
        output_dir: CAPS directory to contain the output
        subject_list:  a list containing all the participant_id
        session_list: a list containing all the session_id

    Returns: the related lists based on the tsv files
    """
    import os
    import errno
    from copy import deepcopy as cp
    import subprocess
    from clinica.utils.stream import cprint

    # subject_id, subject_list and session_list
    subject_id = list(subject_list[i] + '_' + session_list[i] for i in range(len(subject_list)))
    subject_id_without_reconalled = cp(subject_id)
    subject_list_without_reconalled = cp(subject_list)
    session_list_without_reconalled = cp(session_list)

    # output_path is the path to CAPS
    output_path = os.path.expanduser(output_dir)  # change the relative path to be absolute path
    output_dir = os.path.join(output_path, 'subjects')

    try:
        os.makedirs(output_dir)
    except OSError as exception:
        if exception.errno != errno.EEXIST:  # if the error is not exist error, raise, otherwise, pass
            raise

    # subject_dir is the real path to FreeSurfer output path
    subject_dir = []
    subject_dir_without_reconalled = []

    for i in range(len(subject_list)):
        subject = os.path.join(output_dir,
                               subject_list[i],
                               session_list[i],
                               't1',
                               'freesurfer_cross_sectional')
        try:
            os.makedirs(subject)
        except OSError as exception:
            if exception.errno != errno.EEXIST:
                raise

        # add the FS path into a list with all subjects and check the recon-all.log file to see if this subject have been run recon-all successfully.
        subject_dir.append(subject)
        subject_path = os.path.join(subject, subject_id[i])
        subject_path_abs = os.path.expanduser(subject_path)
        # check the recon-all.log
        log_file = os.path.join(subject_path_abs, 'scripts', 'recon-all.log')
        if os.path.isfile(log_file):
            last_line = subprocess.check_output(['tail', '-1', log_file])
            if b'finished without error' in last_line:
                cprint("Skipping %s (FreeSurfer segmentation without error was found)" % subject_id[i])
                subject_id_without_reconalled.remove(subject_id[i])
                subject_list_without_reconalled.remove(subject_list[i])
                session_list_without_reconalled.remove(session_list[i])
            else:
                subject_dir_without_reconalled.append(subject)
        else:
            subject_dir_without_reconalled.append(subject)

    return subject_dir, subject_id, subject_dir_without_reconalled, subject_id_without_reconalled, subject_list_without_reconalled, session_list_without_reconalled


def check_fov(t1_list, recon_all_args):
    """
    Check size of inputs and field of view (FOV) of each T1 image.

    Args:
        t1_list: a list containing all the t1 images
        recon_all_args: default the -qcache flag

    Returns:
        A list containing "<recon_all_args> -cw256" or "<recon_all_args>"
        based on the FOV.
    """
    import sys
    import nibabel as nib
    from nipype.utils.filemanip import filename_to_list
    from clinica.utils.stream import cprint

    output_flags = []
    num_t1 = len(t1_list)
    t1_list = filename_to_list(t1_list)

    if num_t1 == 0:
        cprint("ERROR: No T1's Given")
        sys.exit(-1)

    f = nib.load(t1_list[0])
    voxel_size = f.header.get_zooms()
    t1_size = f.header.get_data_shape()
    if (voxel_size[0] * t1_size[0] > 256) or \
            (voxel_size[1] * t1_size[1] > 256) or \
            (voxel_size[2] * t1_size[2] > 256):
        # cprint("Setting MRI Convert to crop images to 256 FOV")
        optional_flag = '-cw256'
    else:
        # cprint("No need to add -cw256 flag")
        optional_flag = ''
    flag = "{0} ".format(recon_all_args) + optional_flag
    output_flags.append(flag)

    return output_flags


def create_flags_str(input_flags):
    """
    Create a commandline string from a list of input flags

    Args:
        input_flags: the added flag for recon-all command

    Returns: converted string flag
    """
    output_str = ""
    for flag in input_flags:
        output_str += "{0} ".format(flag)
    output_str.strip()  # stripped from the beginning and the end of the string (default whitespace characters).

    return output_str


def write_tsv_files(subject_id, output_dir):
    """
    Generate statistics TSV files for a given subject.
    """
    import os
    from clinica.utils.freesurfer import generate_regional_measures

    participant_id = subject_id.split('_')[0]
    session_id = subject_id.split('_')[1]

    path_segmentation = os.path.join(
        os.path.expanduser(output_dir),
        'subjects',
        participant_id,
        session_id,
        't1',
        'freesurfer_cross_sectional'
    )

    generate_regional_measures(path_segmentation, subject_id)


def print_begin_pipeline(subject_id):
    """
    Display begin message for a given subject.

    Args:
        subject_id: Subject ID (e.g. sub-CLNC01_ses-M0)
    """
    from clinica.utils.stream import cprint
    import datetime
    from colorama import Fore

    now = datetime.datetime.now().strftime('%H:%M:%S')

    cprint('%s[%s]%s Running pipeline for %s...' % (
        Fore.BLUE, now, Fore.RESET, subject_id))


def print_end_pipeline(subject_id, final_file):
    """
    Display end message for a given subject.

    Args:
        subject_id: Subject ID (e.g. sub-CLNC01_ses-M0)
        final_file: One of the last file generated by the pipeline.
            Connect a final file to this argument in order to display the end
            message.
    """
    from clinica.utils.stream import cprint
    import datetime
    from colorama import Fore

    now = datetime.datetime.now().strftime('%H:%M:%S')
    cprint('%s[%s]%s ...%s has completed.' % (
        Fore.GREEN, now, Fore.RESET, subject_id))



# coding: utf8

__author__ = "Simona Bottani"
__copyright__ = "Copyright 2016-2019 The Aramis Lab Team"
__credits__ = ["Simona Bottani"]
__license__ = "See LICENSE.txt file"
__version__ = "0.1.0"
__maintainer__ = "Simona Bottani"
__email__ = "simona.bottani@icm-institute.org"
__status__ = "Development"


def atlas_statistics(file_list, atlas_list):
    """
    For each atlas name provided it calculates for the input image the mean for each region in the atlas and saves it to a TSV file.
    :param in_image: A Nifti image
    :param in_atlas_list: List of names of atlas to be applied
    :return: List of paths to tsv files
    """
    from os.path import abspath, join
    from nipype.utils.filemanip import split_filename
    from clinica.utils.atlas import AtlasAbstract
    from clinica.utils.statistics import statistics_on_atlas
    from clinica.utils.stream import cprint

    orig_dir, base, ext = split_filename(file_list)
    atlas_classes = AtlasAbstract.__subclasses__()
    atlas_statistics_list = []
    for atlas in atlas_list:
        for atlas_class in atlas_classes:
            if atlas_class.get_name_atlas() == atlas:
                out_atlas_statistics = abspath(
                    join('./' + base + '_space-' + atlas + '_map-graymatter_statistics.tsv'))
                cprint(out_atlas_statistics)
                statistics_on_atlas(file_list, atlas_class(), out_atlas_statistics)
                atlas_statistics_list.append(out_atlas_statistics)
    return atlas_statistics_list
